@@ -468,7 +468,8 @@ function make_distribution(seq_len, motif) {
     canvas.height = 90;
     var rset = new CentrimoRSet(seq_len);
     rset.add("", "", "#000", motif["len"], motif["centrimo_total_sites"], motif["centrimo_sites"]);
-    var graph = new CentrimoGraph(rset, triangular_weights(20));
+    var smooth_width = Math.min(20, motif["centrimo_sites"].length);
+    var graph = new CentrimoGraph(rset, triangular_weights(smooth_width));
     add_draw_task(canvas, new DelayCentrimoPlotTask(graph, canvas));
     var link = document.createElement("a");
     link.appendChild(canvas);
