@@ -468,7 +468,8 @@ function make_distribution(seq_len, motif) {
   var rset = new CentrimoRSet(seq_len);
   if (motif["sea_site_distr"]) {
     rset.add("", "", color, motif["len"], motif["sea_total_sites"], motif["sea_site_distr"]);
-    var graph = new CentrimoGraph(rset, triangular_weights(20), null, null, null, null, data["align"], motif["len"]);
+    var smooth_width = Math.min(20, motif["sea_site_distr"].length);
+    var graph = new CentrimoGraph(rset, triangular_weights(smooth_width), null, null, null, null, data["align"], motif["len"]);
     add_draw_task(canvas, new DelayCentrimoPlotTask(graph, canvas));
   }
   return canvas;
