@@ -21,7 +21,7 @@ my $allowed_sym_re = qr/[!-~]/; # Allow all visible ASCII
 my $sym_re = qr/^($allowed_sym_re)$/;
 my $syms_re = qr/^($allowed_sym_re+)$/;
 my $sym_id_re = qr/^([a-zA-Z]|n[0-9]|x[a-zA-Z0-9]{2})$/;
-my $num_list_re = qr/^\s*((\d+\s+)*(\d+))\s*$/;
+my $num_list_re = qr/^\s*(\d[\d\s]*)\s*$/;
 
 my $ST_IN_MOTIF = [
   {ELE => 'pos', RPT => $RPT_ALO, ATRS => [\&_get_alphabet_atrs]}
