@@ -432,7 +432,7 @@ void post_process(CISML_T* cisml, ARRAYLST_T* motifs, bool normalize_scores){
     PATTERN_T* pattern = get_cisml_patterns(cisml)[m_index];
     double maxscore = 1;
     
-    // FIXME: This should be done to the PSSM, not the individual scores!!!
+    // TODO: This should be done to the PSSM, not the individual scores!!!
     // Normalize the scores to RMA format if necessary.
     if (normalize_scores) {
       int k;
@@ -745,6 +745,13 @@ ARRAYLST_T* load_motifs(AMA_OPTIONS_T *opts) {
   while (mread_has_motif(mread)) {
     motif = mread_next_motif(mread);
     total_motifs++;
+    if (get_motif_length(motif) > 300) {
+      if (verbosity >= NORMAL_VERBOSE) {
+        fprintf(stderr, "Warning: Skipping motif %s because its width is > 300 (width= %d).\n", 
+          get_motif_id(motif), get_motif_length(motif));
+      }
+      continue;
+    } 
     if (rbtree_size(opts->selected_motifs) == 0 || rbtree_find(opts->selected_motifs, get_motif_id(motif)) != NULL) {
       if (verbosity >= HIGH_VERBOSE) {
         fprintf(stderr, "Using motif %s of width %d.\n", get_motif_id(motif), get_motif_length(motif));
@@ -1017,11 +1024,11 @@ int main(int argc, char **argv) {
       exit(1);
     }
     path = make_path_to_file(options.out_dir, text_filename);
-    //FIXME check for errors: MEME doesn't either and we at least know we have a good directory
+    //TODO check for errors: MEME doesn't either and we at least know we have a good directory
     text_output = fopen(path, "w");
     free(path);
     path = make_path_to_file(options.out_dir, cisml_filename);
-    //FIXME check for errors
+    //TODO check for errors
     cisml_output = fopen(path, "w");
     free(path);
     print_cisml(cisml_output, cisml, true, NULL, false);
