@@ -8,7 +8,7 @@
  * dynamic programming, which dictates a specific approach to some pY
  * calculations.
  *
- * $Id: calculate_p_y.c 3915 2009-07-21 07:10:21Z tbailey $
+ * $Id$
  *
  * $Log$
  * Revision 1.1.2.2  2006/02/09 23:51:15  twhitington
@@ -44,23 +44,24 @@ void get_pY(
   int n_samples = dataset->n_samples;
   SAMPLE **samples = dataset->samples;
   
-  for (i=0; i<n_samples; i++) {         /* sequence */
+  for (i=0; i<n_samples; i++) { // sequence 
     SAMPLE *s = samples[i];
     int lseq = s->length;
-    char *res = pYindex<2 ? s->res : s->resic;  /* integer sequence */
-    int *pY = s->pY[pYindex];           /* log p(Y_j | theta_1) */
+    uint8_t *res = (pYindex < 2 ? s->res : s->resic);	// integer sequence 
+    int *pY = s->pY[pYindex]; // log p(Y_j | theta_1) 
 
-    if (lseq < w) continue;             /* skip if sequence too short */
+    if (lseq < w) continue; // skip if sequence too short 
+    skip_group_if_required(dataset, s, i);		// don't score holdout groups
 
-    for (j=0; j<=lseq-w; j++) {         /* site start */
-      char *r = res + j;
+    for (j=0; j<=lseq-w; j++) { // site start 
+      uint8_t *r = res + j;
       int p = 0;
-      for (k=0; k<w; k++) {             /* position in site */
-        p += theta_1[k][(int) (*r++)];
+      for (k=0; k<w; k++) { // position in site 
+        p += theta_1[k][(*r++)];
       }
       pY[j] = p;
     }
-    for (j=lseq-w+1; j<lseq; j++) pY[j] = 0;    /* impossible positions */
+    for (j=lseq-w+1; j<lseq; j++) pY[j] = 0; // impossible positions 
   }
 } // get_pY
 
@@ -68,40 +69,44 @@ void get_pY(
 /**
  * combine_strands
  *
- * This function operates on an array of SAMPLE objects. For each sample, the
+ * This function operates on an array of SAMPLE objects contained in a
+ * DATASET object. For each sample, the
  * first pY scratch array is instantiated to contain the best pY values from
  * the positive and negative strands.
  *
  */
-void combine_strands(
-  SAMPLE **samples,  ///< An array of pointers to SAMPLE objects, whose first
-                     ///< scratch pY arrays will be modified.
-  int n_samples,     ///< The length of the array of SAMPLE pointers.
+void inline combine_strands(
+  DATASET *dataset,
   int w              ///< The length of the motifs under which the pY
-                     ///< arrays were initially instaniated.
+                     ///< arrays were initially instantiated.
 )
 {
+  SAMPLE **samples = dataset->samples;
+  int n_samples = dataset->n_samples;
   int i;
-  for (i=0; i<n_samples; i++) { /* sequence */
+  for (i=0; i<n_samples; i++) { // sequence 
     SAMPLE *s = samples[i];
     int lseq = s->length;
-    int *pY = s->pY[0];                 	/* log p(Y_j | theta_1) both */
-    int *pY1 = s->pY[1];                        /* log p(Y_j | theta_1) + strand */
-    int *pY2 = s->pY[2];                        /* log p(Y_j | theta_1) - strand */
-    char *pYic = s->pYic;                       /* site on - strand */
-    int last_j = lseq-w;                        /* last possible site */
+    int *pY = s->pY[0]; // log p(Y_j | theta_1) both 
+    int *pY1 = s->pY[1]; // log p(Y_j | theta_1) + strand 
+    int *pY2 = s->pY[2]; // log p(Y_j | theta_1) - strand 
+    char *pYic = s->pYic; // site on - strand 
+    int last_j = lseq-w; // last possible site 
     
-    if (lseq < w) continue;             /* skip if sequence too short */
+    if (lseq < w) continue; // skip if sequence too short 
+    skip_group_if_required(dataset, s, i);		// don't score holdout groups
     
     int j, k;
-    for (j=0, k=last_j; j<=last_j; j++, k--) {  /* site start */
-      if (pY2[k] > pY1[j]) {            /* site on - strand */
-        pYic[j] = '\1'; pY[j] = pY2[k];
-      } else {                          /* site on + strand */
-        pYic[j] = '\0'; pY[j] = pY1[j]; 
+    for (j=0, k=last_j; j<=last_j; j++, k--) { // site start
+      if (pY2[k] > pY1[j]) { // site on - strand
+        pYic[j] = '\1';
+        pY[j] = pY2[k];
+      } else { // site on + strand 
+        pYic[j] = '\0';
+        pY[j] = pY1[j];
       }
-    } /* site start */
-  } /* sequence */
+    } // site start 
+  } // sequence 
 } // Combine strands
 
 
@@ -121,26 +126,27 @@ void combine_strands(
  * of int pointers, lmotif, so that each pointer points to an array in lmap.
  *
  */
-extern void create_lmotif (
+void create_lmotif (
+  ALPH_T *alph,
   char *seed_str,    ///< The seed as a string
   int lmap[MAXALPH][MAXALPH], ///< Matrix representing a seq-to-theta mapping fn
   int *lmotif[MAXSITE], ///< The resulting lmotif - OUT
   int *mot_width     ///< The width of the motif - OUT
 ) {
   // Retrieve the integer-encoded representation of the seed:
-  char *seed_enc = NULL;
+  uint8_t *seed_enc = NULL;
   int seed_len;
-  seed_enc = to_e_seed(seed_str, &seed_len);
+  seed_enc = to_e_seed(alph, seed_str, &seed_len);
 
   // Allocate space for the lmotif data structure:
 /*   int **columns = NULL; */
 /*   Resize(columns, seed_len, int *); */
-/*   int col_idx; */
-/*   for (col_idx = 0; col_idx < seed_len; col_idx++) { */
+ // int col_idx; 
+ // for (col_idx = 0; col_idx < seed_len; col_idx++) { 
 /*     int *curr_col; */
-/*     Resize(curr_col, ALPH, int); */
-/*     columns[col_idx] = curr_col; */
-/*   } */
+ // Resize(curr_col, ALPH, int); 
+ // columns[col_idx] = curr_col; 
+ // } 
 
   // Convert the encoded seed to an lmotif, thus filling in the memory
   // allocated for the "lmotif" data structure:
@@ -160,15 +166,15 @@ extern void create_lmotif (
  * Memory must be pre-allocated for theta_1.
  *
  */
-extern void init_lmotif (
+void init_lmotif (
   int w,             ///< width of site
-  char *res,         ///< Integer encoded letters of subsequence
+  uint8_t *res,         ///< Integer encoded letters of subsequence
   int *theta_1[MAXSITE], ///< theta_1
   int lmap[MAXALPH][MAXALPH] ///< matrix of frequency vectors
 ) {
   int m;
   for (m=0; m<w; m++) {
-    theta_1[m] = lmap[(int)res[m]];
+    theta_1[m] = lmap[res[m]];
   }
 } // init_lmotif
 
@@ -189,14 +195,14 @@ extern void init_lmotif (
  * substring search, although this approach has not yet been implemented.
  *
  */
-extern void evaluate_seed_DP (
+void evaluate_seed_DP (
   char *new_seed,    ///< The string representing the new seed object.
   SEED_DIFFS *s_diffs, ///< An object representing the differences between the
                      ///< seed of interest and the "previous" seed.
   int lmap[MAXALPH][MAXALPH], ///< Matrix representing a seq-to-theta mapping fn
-  MTYPE mtype,       ///< The type of sequence model evaluation is performed
+  MOTYPE mtype,       ///< The type of sequence model evaluation is performed
                      ///< under.
-  BOOLEAN ic,        ///< Consider inverse complement in DNA.
+  bool ic,        ///< Consider inverse complement in DNA.
   DATASET *dataset,  ///< The dataset, containing the sequences used during
                      ///< evaluation of the current seed.
   SP_MATRIX *sp_mat  ///< A matrix of starting points, into which the evaluated
@@ -210,7 +216,7 @@ extern void evaluate_seed_DP (
   // next_pY_branching => Obtain it now:
   int *lmotif[MAXSITE];
   int mot_width;
-  create_lmotif(new_seed, lmap, lmotif, &mot_width);
+  create_lmotif(dataset->alph, new_seed, lmap, lmotif, &mot_width);
   assert(mot_width == new_seed_len);
 
   // Update the array of pY values given the new seed. This is where the
@@ -218,7 +224,6 @@ extern void evaluate_seed_DP (
   if (!ic) {
     //fprintf(stderr, "NPY....\n");
     next_pY_branching(lmotif, s_diffs, dataset, 0);
-    //get_pY(dataset, lmotif, s_diffs->length_new, 0);
   } else {
     next_pY_branching(lmotif, s_diffs, dataset, 1);
     next_pY_branching(lmotif, s_diffs, dataset, 2);
@@ -227,15 +232,13 @@ extern void evaluate_seed_DP (
   /* Combine the log Pr(Y_ij | theta_1) values from each strand if using
    * the inverse complement:
    */
-  if (ic) {
-    combine_strands(dataset->samples, dataset->n_samples, new_seed_len);
-  }
+  if (ic) combine_strands(dataset, new_seed_len);
   
   // Get a list of "maxima" under the specified sequence model. These maxima
   // will indicate where the pY values are highest:
   int n_maxima = ps(dataset, new_seed_len); // upper bound on # maxima
-  P_PROB maxima = (P_PROB) mymalloc(n_maxima * sizeof(p_prob));
-  n_maxima = get_max(mtype, dataset, new_seed_len, maxima, ic, TRUE);
+  P_PROB maxima = (P_PROB) mm_malloc(n_maxima * sizeof(p_prob));
+  n_maxima = get_max(mtype, dataset, false, new_seed_len, 0, maxima, ic, true);
   
   // Obtain the S_POINT array of the new seed's length, from the s_point matrix:
   S_POINT *sp_arr = get_sp_arr(sp_mat, new_seed_len);
@@ -246,7 +249,7 @@ extern void evaluate_seed_DP (
      array... */
   int iseq = -1;     // Dummy
   int ioff = -1;     // Dummy
-  char *eseq = NULL; // Dummy
+  uint8_t *eseq = NULL; // Dummy
   char *name = NULL; // Dummy
   double col_scores[MAXSITE]; // Dummy
   // Force align_top_subsequences to set the score of every s_point:
@@ -265,6 +268,7 @@ extern void evaluate_seed_DP (
     name,
     n_nsites,
     n_maxima,
+    0,
     maxima,
     col_scores,
     sp_arr
@@ -274,7 +278,7 @@ extern void evaluate_seed_DP (
   myfree(maxima);
   
   // Update the heap in each of the relevant S_POINTs, adding a new "SEED"
-  // object with the appropriate score the the S_POINT's heap in each case:
+  // object with the appropriate score to the S_POINT's heap in each case:
   update_s_point_heaps(sp_arr, new_seed, n_nsites);
 
 } // evaluate_seed_DP
@@ -300,7 +304,7 @@ extern void evaluate_seed_DP (
  * substring search, although this approach has not yet been implemented.
  *
  */
-extern void next_pY_branching(
+void next_pY_branching(
   int *lmotif_new[MAXSITE], ///< The lmotif corresponding to the new seed.
   SEED_DIFFS *s_diffs, ///< The differences between the new and old seeds.
   DATASET *dataset,  ///< The dataset of sequences. Contains the pY arrays.
@@ -361,10 +365,11 @@ extern void next_pY_branching(
       SAMPLE *curr_seq = samples[seq_idx]; // The current sequence
       int lseq = curr_seq->length; // Length of current sequence
       // Integer encoding of the sequence:
-      char *res = pYindex<2 ? curr_seq->res : curr_seq->resic;
+      uint8_t *res = (pYindex < 2 ? curr_seq->res : curr_seq->resic);
       int *pY = curr_seq->pY[pYindex]; // log p(Y_j | theta_1)
       
       if (lseq < new_seed_len) continue; // Skip if sequence is too short
+      skip_group_if_required(dataset, curr_seq, seq_idx);		// don't score holdout groups
       
       // Shift the sequence (adjusting according to the indeces at which the
       // current and previous motifs differ) now, rather than inside inner
@@ -372,8 +377,8 @@ extern void next_pY_branching(
 
       // Residues - for the current sequence position - at which the new and old
       // seeds differ:
-      char *diff_1 = res+diff_idx1;
-      char *diff_2 = NULL;
+      uint8_t *diff_1 = res+diff_idx1;
+      uint8_t *diff_2 = NULL;
       if (n_diffs == 2) {
         diff_2 = res+diff_idx2;
       }
@@ -388,7 +393,7 @@ extern void next_pY_branching(
            * pY value by the amount indicated in the difference column for the
            * letter at that column in the current subsequence...
            */
-          pY[lett_idx] = pY[lett_idx] + diff_col1[(int)(*diff_1++)];
+          pY[lett_idx] = pY[lett_idx] + diff_col1[(*diff_1++)];
         } else {
           /* There are two columns differing between this motif and the previous
            * motif evaluated. Evaluate each pY taking this into consideration.
@@ -398,8 +403,8 @@ extern void next_pY_branching(
            */
           pY[lett_idx] =
             pY[lett_idx] +
-            diff_col1[(int)(*diff_1++)] +
-            diff_col2[(int)(*diff_2++)];
+            diff_col1[(*diff_1++)] +
+            diff_col2[(*diff_2++)];
         } // examining col_b_idx
       } // offset lett_idx
     } // sequence seq_idx
@@ -413,10 +418,11 @@ extern void next_pY_branching(
       SAMPLE *curr_seq = samples[seq_idx]; // The current sequence
       int lseq = curr_seq->length; // Length of current sequence
       // Integer encoding of the sequence:
-      char *res = pYindex<2 ? curr_seq->res : curr_seq->resic;
+      uint8_t *res = (pYindex < 2 ? curr_seq->res : curr_seq->resic);
       int *pY = curr_seq->pY[pYindex]; // p(Y_j | theta_1)
       
       if (lseq < new_seed_len) continue; // Skip if sequence is too short
+      skip_group_if_required(dataset, curr_seq, seq_idx);		// don't score holdout groups
       
       // Shift the sequence (adjusting according to the indeces at which the
       // current and previous motifs differ) now, rather than inside inner
@@ -424,8 +430,8 @@ extern void next_pY_branching(
 
       // Residues - for the current sequence position - at which the new and old
       // seeds differ:
-      char *diff_1 = res+diff_idx1;
-      char *diff_2 = NULL;
+      uint8_t *diff_1 = res+diff_idx1;
+      uint8_t *diff_2 = NULL;
       if (n_diffs == 2) {
         diff_2 = res+diff_idx2;
       }
@@ -443,7 +449,7 @@ extern void next_pY_branching(
            */
           // (n_diffs == 1) && (shift < 0) => The difference is the single
           // shift. pY[ij] is set using pY[i(j+1)] under the old seed:
-          pY[lett_idx] = pY_shifted[lett_idx] + diff_col1[(int)(*diff_1++)];
+          pY[lett_idx] = pY_shifted[lett_idx] + diff_col1[(*diff_1++)];
         } else {
           /* There are two columns differing between this motif and the previous
            * motif evaluated. Evaluate each pY taking this into consideration.
@@ -455,8 +461,8 @@ extern void next_pY_branching(
           // (between seeds) are accounted for by the shift:
           pY[lett_idx] =
             pY_shifted[lett_idx] +
-            diff_col1[(int)(*diff_1++)] +
-            diff_col2[(int)(*diff_2++)];
+            diff_col1[(*diff_1++)] +
+            diff_col2[(*diff_2++)];
         } // examining n_diffs
       } // offset lett_idx
     } // sequence seq_idx
@@ -471,10 +477,11 @@ extern void next_pY_branching(
       SAMPLE *curr_seq = samples[seq_idx]; // The current sequence
       int lseq = curr_seq->length; // Length of current sequence
       // Integer encoding of the sequence:
-      char *res = pYindex<2 ? curr_seq->res : curr_seq->resic;
+      uint8_t *res = (pYindex < 2 ? curr_seq->res : curr_seq->resic);
       int *pY = curr_seq->pY[pYindex]; // p(Y_j | theta_1)
       
       if (lseq < new_seed_len) continue; // Skip if sequence is too short
+      skip_group_if_required(dataset, curr_seq, seq_idx);		// don't score holdout groups
 
       // Shift the sequence (adjusting according to the indeces at which the
       // current and previous motifs differ) now, rather than inside inner
@@ -483,8 +490,8 @@ extern void next_pY_branching(
       int first_char_pos = (lseq-new_seed_len-n_end_pos);
       // Residues - for the current sequence position - at which the new and old
       // seeds differ:
-      char *diff_1 = res+diff_idx1+first_char_pos;
-      char *diff_2 = NULL;
+      uint8_t *diff_1 = res+diff_idx1+first_char_pos;
+      uint8_t *diff_2 = NULL;
       if (n_diffs == 2) {
         diff_2 = res+diff_idx2+first_char_pos;
       }
@@ -497,12 +504,12 @@ extern void next_pY_branching(
           lett_idx--) {
         if (n_diffs == 1) {
           // Only 1 column differs compared with the previous motif...
-          pY[lett_idx] = pY_shifted[lett_idx] + diff_col1[(int)(*diff_1--)];
+          pY[lett_idx] = pY_shifted[lett_idx] + diff_col1[(*diff_1--)];
         } else {
           pY[lett_idx] =
             pY_shifted[lett_idx] +
-            diff_col1[(int)(*diff_1--)] +
-            diff_col2[(int)(*diff_2--)];
+            diff_col1[(*diff_1--)] +
+            diff_col2[(*diff_2--)];
         } // examining n_diffs
       } // lett_idx
     } // seq_idx
@@ -521,10 +528,11 @@ extern void next_pY_branching(
     SAMPLE *curr_seq = samples[seq_idx]; // The current sequence
     int lseq = curr_seq->length; // Length of current sequence
     // Integer encoding of the sequence:
-    char *res = pYindex<2 ? curr_seq->res : curr_seq->resic;
+    uint8_t *res = (pYindex < 2 ? curr_seq->res : curr_seq->resic);
     int *pY = curr_seq->pY[pYindex]; // p(Y_j | theta_1)
     
     if (lseq < new_seed_len) continue; // Skip if sequence is too short
+    skip_group_if_required(dataset, curr_seq, seq_idx);		// don't score holdout groups
 
     // Calculate pY values at the start of the sequence:
     int lett_idx;
@@ -534,9 +542,9 @@ extern void next_pY_branching(
       int mot_idx; // Index within the lmotif for the new seed.
       // Retrieve the log prob from the letter for each position in the site:
       for (mot_idx = 0; mot_idx < new_seed_len; mot_idx++) {
-/*       for (mot_idx = 0; mot_idx < (lett_idx + new_seed_len); mot_idx++) { */
-        char curr_lett = res[lett_idx + mot_idx]; // Current letter in seq
-        extra_pY_value += lmotif_new[mot_idx][(int)curr_lett];
+      // for (mot_idx = 0; mot_idx < (lett_idx + new_seed_len); mot_idx++) { 
+        uint8_t curr_lett = res[lett_idx + mot_idx]; // Current letter in seq
+        extra_pY_value += lmotif_new[mot_idx][curr_lett];
       }
       pY[lett_idx] = extra_pY_value;
     } // lett_idx
@@ -550,8 +558,8 @@ extern void next_pY_branching(
       int mot_idx; // Index within the lmotif for the new seed.
       // Retrieve the log prob from the letter for each position in the site:
       for (mot_idx = 0; mot_idx < new_seed_len; mot_idx++) {
-        char curr_lett = res[lett_idx + mot_idx]; // Current letter in seq
-        extra_pY_value += lmotif_new[mot_idx][(int)curr_lett];
+        uint8_t curr_lett = res[lett_idx + mot_idx]; // Current letter in seq
+        extra_pY_value += lmotif_new[mot_idx][curr_lett];
       }
       pY[lett_idx] = extra_pY_value;
     } // lett_idx
