@@ -80,7 +80,7 @@ static int TRACE = 0;        /* turn off debug trace */
 // Linear:
 #define MU1(gc, a, b) ((a) + (b)*(gc))
 
-// FIXME TLB: for score-set
+// TODO TLB: for score-set
 char *OUTPUT_DIRNAME = "mcast_out";
 
 /******************************************************************************/
@@ -133,7 +133,7 @@ static void init_score_set(SCORE_SET *score_set) {
 }
 
 #ifdef DEBUG_SCORE
-// FIXME CEG
+// TODO CEG
 void print_score_set(DISTR_T dtype, SCORE_SET score_set, EVD evd) {
   int num_scores = score_set.n;
   int g = evd.g;
@@ -422,7 +422,7 @@ SCORE_SET *set_up_score_set(
       score_set->e_hit_score = e_hit_score;
 
     } else {          // LINEAR_HMM
-      // FIXME: need to figure out E[span] for LINEAR
+      // TODO: need to figure out E[span] for LINEAR
       // models but I haven't thought about it.
     }
  
@@ -642,11 +642,6 @@ static EVD em_exp(
       sxz += z_i * x_i;					// Sum x_i z_i
       szx2 += (1 - z_i) * (x_i - mu2) * (x_i - mu2);	// Sum (1-z_i)(x_i-mu2)^2
       L += log( c * f1 + (1-c) * f2 );
-//FIXME
-if (isnan(L)) {
-fprintf(stderr, "L = %g f1 %g f2 %g c %g mu1 %g g_i %g\n", L, f1, f2, c, mu1, scores->gc);
-exit(0);
-}
     } // scores
 
     // Done if likelihood didn't change.
@@ -857,7 +852,7 @@ static EVD_SET fitexp(
         gc_mid = evd.mid_t;		// Save new mean GC.
         gc_hi = evd.max_t;		// Save new hi GC.
 #ifdef DEBUG_SCORE
-        // FIXME CEG
+        // TOD CEG
         print_score_set(evd_set.dtype, score_set, evd);
 #endif
         // Flag problem if exponential mean greater than Gaussian: fitting wrong tail.
@@ -877,7 +872,7 @@ static EVD_SET fitexp(
         evds[g] = evd;
         i_low = i+1;
 #ifdef DEBUG_SCORE
-        // FIXME CEG
+        // TODO CEG
         if (dtype==D_EXP) {
           fprintf(
             stderr, 
@@ -2057,7 +2052,7 @@ int main(int argc, char **argv)
     i++;
   }
 
-  //FIXME: add maxscores to fit_distribution
+  //TODO: add maxscores to fit_distribution
 
   /* check input arguments */
   if (H > MAXH) {        /* check that H is in range */
