@@ -319,7 +319,7 @@ int main(
         int end = length - (xlate_src_nsyms(xlate) - 1);
         int strands = (alph_has_complement(xlate_src_alph(xlate)) ? 2 : 1);
         for (i = 0; i < strands; i++) { // positive then negative strands
-          if (i) invcomp_seq(xlate_src_alph(xlate), seq, length); // negative strand
+          if (i) invcomp_seq(xlate_src_alph(xlate), seq, length, false); // negative strand
           for (j = 0; j < xlate_src_nsyms(xlate); j++) { // frame
             int f = j + (xlate_src_nsyms(xlate) * i);
             for (k = j; k < end; k += xlate_src_nsyms(xlate)) {
