@@ -400,6 +400,8 @@ Sint read_fasta_to_multiseqs(
   Uint totallength = 0;		// total length of sequences in multiseq object
   double *freqs = (double *) calloc(alen, sizeof(double));
   Uint nletters = 0;
+  int n_seqs = 0;
+  char *db_type = "primary";
   for (i=i_seq=0; i<nseqs; i++, i_seq++) {
     // The first ntrain (shuffled order) sequences are training; rest are test.
     Multiseq *multiseq = (i < ntrain) ? train_multiseq : test_multiseq;
@@ -408,10 +410,20 @@ Sint read_fasta_to_multiseqs(
       newptr = multiseq->sequence;
       i_seq = 0;
       totallength = 0;
+      n_seqs = 0;
+      if (i==ntrain) db_type = "control";
     }
     sptr = *(PEEKARRAY(&seqptrs, permuted_indices[i], Seqptr));
     // Skip sequence if it would cause total length to be exceeded.
-    if (maxlength > 0 && totallength + sptr.seqlen > maxlength) continue;
+    if (maxlength > 0 && totallength + sptr.seqlen > maxlength) {
+      DEBUG_FMT(QUIET_VERBOSE, 
+        "# Warning: Skipping %s sequence '>%*.*s' of length %d because\n"
+        "#          it causes total database length to be exceeded (%d > %d).\n",
+        db_type, sptr.idlen, sptr.idlen, sptr.id, sptr.seqlen, totallength + sptr.seqlen, maxlength);
+      continue;
+     } else {
+      n_seqs++;
+     }
     // Update the number of sequences stored and their total length.
     multiseq->numofsequences++;
     totallength += sptr.seqlen;
@@ -422,7 +434,7 @@ Sint read_fasta_to_multiseqs(
     }
     STOREINARRAY(&multiseq->descspace, Uchar, 4096, (Uchar) '\0');
     // Place the separator and store the markpos (except for first sequence).
-    if (i != 0 && i != ntrain) {
+    if (n_seqs != 1) {
       STOREINARRAY(&multiseq->markpos, Uint, 128, (Uint) (newptr - multiseq->sequence));
       *newptr++ = SEPARATOR;
     }
