@@ -70,7 +70,7 @@ static inline LO* convert_motif(ARRAY_T *bgfreqs, int alen, int nsyms, MOTIF_T *
   // create reverse complement of best sequence if complementable motif 
   if (alph_has_complement(lo->alph)) {                              // DNA motif: reverse comp. 
     strcpy(lo->best_icseq, lo->best_seq);
-    invcomp_seq(lo->alph, lo->best_icseq, lo->w);
+    invcomp_seq(lo->alph, lo->best_icseq, lo->w, false);
   } else { // if not complementable then just reverse 
     for (i=0; i<lo->w; i++) lo->best_icseq[i] = lo->best_seq[lo->w-1-i];
     lo->best_icseq[i] = '\0';
