@@ -452,7 +452,7 @@ int meme_main(
     snprintf(
       stopping_reason,
       MAX_REASON_LENGTH,
-      "Stopped because motif E-value > %8.2e.",
+      "Stopped because the next motif E-value > %8.2e.",
       dataset->evt
     );
   }
@@ -583,7 +583,11 @@ int meme_main(
 #ifdef DEBUG_PARALLEL
   fprintf(stderr, "%d: At mpFinalize\n", mpMyID()); fflush(stderr);
 #endif
-  if (mpi) mpFinalize();
+  if (mpi) {
+    MPI_Type_free(SEED_PACKET_TYPE_PTR);
+    MPI_Type_free(REDUCTION_PACKET_TYPE_PTR);
+    mpFinalize();
+  }
 #endif
 
   return (output_error ? 1 : 0);
