@@ -170,7 +170,7 @@ void print_mhmmscan_xml_file(
     num_seqs,
     num_residues
   );
-  //TODO FIXME make work with other alphabets
+  //TODO make work with other alphabets
   fprintf(
     out,
     "<alphabet>%s</alphabet>\n",
@@ -565,7 +565,7 @@ static SCORE_SET *read_and_score(
   //
   // Free structures.
   //
-  if (0) { // FIXME
+  if (0) { // TODO
     free_matrix(dp_matrix);
     free_matrix(trace_matrix);
     free_match(complete_match);
@@ -1299,7 +1299,7 @@ void process_command_line(
       die("You may only specify a positive min-score.\n");
     }
 
-    // FIXME: both-strands not implemented
+    // TODO: both-strands not implemented
     if (options->both_strands) {
       die("Sorry, -both-strands not yet implemented.");
     }
