@@ -186,6 +186,9 @@ MOTIF_DB_T *read_motifs_and_background(
       if (set_conv_alph) {
 	// The alphabet does not match but we have been told to convert it so
 	// we must check that such a thing is possible.
+        if (db_alph == NULL) {
+	  die("The motifs in '%s' have an unrecognized alphabet and conversion was requested.\n", motif_source);
+        }
 	switch(alph_core_subset(db_alph, alphabet)) {
 	  case 0:
 	    die("The motifs in '%s' have the '%s' alphabet which is not "
