@@ -87,6 +87,7 @@ struct patterns {
   regex_t bg_source;
   regex_t freq_pair;
   regex_t whitespace;
+  regex_t comment_line;
   regex_t motif_intro;
   regex_t motif_id2;
   regex_t motif_kv_pair;
@@ -148,6 +149,7 @@ static const char * BG_SOURCE_RE = "^[[:space:]]+\\(from[[:space:]]+"
 static const char * FREQ_PAIR_RE = "^[[:space:]]*([A-Za-z0-9\\.\\*\\-])[[:space:]]+"
     POS_NUM_RE "([[:space:]].*)?$";
 static const char * WHITESPACE_RE = "^[[:space:]]*$";
+static const char * COMMENT_LINE_RE = "^[[:space:]]*#";
 // Can't ignore case on this one as normal meme format repeatedly
 // uses Motif as the first word on a line
 static const char * MOTIF_INTRO_RE = "^[[:space:]]*(BL[[:space:]]+)?MOTIF"
@@ -200,6 +202,7 @@ static void compile_patterns(struct patterns *re) {
   regcomp_or_die("bg source", &(re->bg_source), BG_SOURCE_RE, REG_EXTENDED | REG_ICASE);
   regcomp_or_die("freq pair", &(re->freq_pair), FREQ_PAIR_RE, REG_EXTENDED);
   regcomp_or_die("whitespace", &(re->whitespace), WHITESPACE_RE, REG_NOSUB);
+  regcomp_or_die("comment line", &(re->comment_line), COMMENT_LINE_RE, REG_NOSUB);
   regcomp_or_die("motif intro", &(re->motif_intro), MOTIF_INTRO_RE, REG_EXTENDED);
   regcomp_or_die("motif id 2", &(re->motif_id2), MOTIF_ID2_RE, REG_EXTENDED);
   regcomp_or_die("motif kv pair", &(re->motif_kv_pair), MOTIF_KV_PAIR_RE, REG_EXTENDED);
@@ -237,6 +240,7 @@ static void free_patterns(struct patterns *re) {
   regfree(&(re->bg_source));
   regfree(&(re->freq_pair));
   regfree(&(re->whitespace));
+  regfree(&(re->comment_line));
   regfree(&(re->motif_intro));
   regfree(&(re->motif_id2));
   regfree(&(re->motif_kv_pair));
@@ -841,9 +845,20 @@ static bool state_in_alphabet(MTEXT_T *parser, const char *line) {
   } else if (regexec_or_die("divider", &(parser->re.divider), line, 4, matches, 0)) {
     // seen line of "****"
     if (parser->counter > 0) done = true;
+  } else if (regexec_or_die("comment line", &(parser->re.comment_line), line, 0, matches, 0)) {
+    // seen line containing whitespace and then "#"
   } else {
     // try to parse line as part of the alphabet
-    alph_reader_line(parser->alph_rdr, line);
+    // after removing any trailing comment text
+    char *trimmed_line = mm_malloc(sizeof(char) * (strlen(line)+1));
+    int i = 0;
+    for (i=0; line[i] != '\0'; i++) {
+      if (line[i] == '#') { trimmed_line[i] = '\0'; break; }
+      trimmed_line[i] = line[i];
+    }
+    trimmed_line[i] = '\0';
+    alph_reader_line(parser->alph_rdr, trimmed_line);
+    free(trimmed_line);
   }
   if (done) alph_reader_done(parser->alph_rdr);
   // report any problems that the alphabet reader finds
