@@ -9,21 +9,21 @@
 extern void read_psp_file(
   char *psp_filename,		/* file containing positional priors */
   DATASET *dataset,		// the dataset
-  BOOLEAN psp_revcomp,          // PSP file contains both strands
-  BOOLEAN meme_revcomp,         // MEME using both strands
-  MTYPE mtype			/* OOPS, ZOOPS or TCM? */
+  bool psp_revcomp,          // PSP file contains both strands
+  bool meme_revcomp,         // MEME using both strands
+  MOTYPE mtype			/* OOPS, ZOOPS or TCM? */
 );
 extern void psp_renormalize(
   DATASET *dataset, 		/* the dataset */
   int new_w,			/* new motif width */
-  BOOLEAN invcomp,		/* reverse complement? */
-  MTYPE mtype			/* OOPS, ZOOPS or TCM? */
+  bool invcomp,		/* reverse complement? */
+  MOTYPE mtype			/* OOPS, ZOOPS or TCM? */
 );
 extern void add_psp_to_log_not_o(
   DATASET *dataset,             /* the dataset */
   int w,                        /* motif width */
-  BOOLEAN invcomp,              /* reverse complement? */
-  MTYPE mtype                   /* model type */
+  bool invcomp,              /* reverse complement? */
+  MOTYPE mtype                   /* model type */
 );
 
 #endif
