@@ -16,6 +16,7 @@
  * gapless alignment of two sequences (ie two seeds). */
 
 static void create_diff_cols (
+  ALPH_T *alph, // alphabet
   char *seed1,     ///< A seed's string representation
   char *seed2,     ///< Another seed's string representation
   int offset,      ///< The number of characters that seed1 is shifted to the
@@ -59,7 +60,8 @@ static void create_diff_cols (
  * \return A SEED_DIFFS object representing the differences between the two
  * seeds under the implicit alignment determined by this function.
  */
-extern SEED_DIFFS *get_seed_diffs (
+SEED_DIFFS *get_seed_diffs (
+  ALPH_T *alph,
   char *old_seed,    ///< String representation of the old seed
   char *new_seed,    ///< String representation of the new seed
   int lmap[MAXALPH][MAXALPH] ///< Log freq x letter map (used for generating
@@ -140,16 +142,16 @@ extern SEED_DIFFS *get_seed_diffs (
      achieved above by "get_n_diffs", but we do it separately instead here,
      because I want to avoid performing unnecessary dynamic memory allocation,
      which is computationally intensive. */
-  int **diff_cols;
-  int *diff_idxs;
-  int n_diffs;
-  create_diff_cols(new_seed, old_seed, new_seed_best_off, lmap,
+  int **diff_cols = NULL;
+  int *diff_idxs = NULL;
+  int n_diffs = 0;
+  create_diff_cols(alph, new_seed, old_seed, new_seed_best_off, lmap,
                    diff_cols, diff_idxs, &diff_cols, &diff_idxs, &n_diffs);
 
   assert(n_diffs == best_n_diffs);
 
   // Create the seed_diffs object:
-  SEED_DIFFS *seed_diffs_obj = (SEED_DIFFS *) mymalloc(sizeof(SEED_DIFFS));
+  SEED_DIFFS *seed_diffs_obj = (SEED_DIFFS *) mm_malloc(sizeof(SEED_DIFFS));
   seed_diffs_obj->old_new_shift = new_seed_best_off;
   
   seed_diffs_obj->diff_cols = diff_cols;
@@ -167,7 +169,7 @@ extern SEED_DIFFS *get_seed_diffs (
  *
  * "Destructor" method: Free the memory associated with this seed diffs object.
  */
-extern void free_seed_diffs (
+void free_seed_diffs (
   SEED_DIFFS *s_diffs ///< "This" seed_diffs object
 ) {
   // Destroy the diff_columns:
@@ -201,7 +203,7 @@ extern void free_seed_diffs (
  *
  * \return The offset
  */
-extern int get_seed_shift(
+int get_seed_shift(
   SEED_DIFFS *diffs  ///< The seed_diffs object
 ) {
   return diffs->old_new_shift;
@@ -219,7 +221,7 @@ extern int get_seed_shift(
  *
  * \return The array of lmotif difference columns
  */
-extern int **get_diff_cols(
+int **get_diff_cols(
   SEED_DIFFS *diffs  ///< The seed_diffs object
 ) {
   return diffs->diff_cols;
@@ -236,7 +238,7 @@ extern int **get_diff_cols(
  *
  * \return An array of indices.
  */
-extern int *get_diff_idxs (
+int *get_diff_idxs (
   SEED_DIFFS *diffs  ///< The seed_diffs object
 ) {
   return diffs->diff_idxs;
@@ -250,7 +252,7 @@ extern int *get_diff_idxs (
  *
  * \return Number of differing characters in the implicit alignment.
  */
-extern int get_n_diffs (
+int get_n_diffs (
   SEED_DIFFS *diffs  ///< The seed_diffs object
 ) {
   return diffs->n_diffs;
@@ -263,7 +265,7 @@ extern int get_n_diffs (
  * Get the lengths of each of the seeds
  *
  */
-extern void get_seed_lengths (
+void get_seed_lengths (
   SEED_DIFFS *diffs, ///< The seed_diffs object
   int *length_old,   ///< The length of the old seed - OUT
   int *length_new    ///< The length of the new seed - OUT
@@ -288,6 +290,7 @@ extern void get_seed_lengths (
  * time.
  */
 static void create_diff_cols (
+  ALPH_T *alph, // alphabet
   char *new_seed,    ///< A seed's string representation
   char *old_seed,       ///< Another seed's string representation
   int offset,        ///< The number of characters that new_seed is shifted to the
@@ -326,8 +329,8 @@ static void create_diff_cols (
   // Get the integer-encoded representation of each of the seeds (offset and
   // non-offset):
   int non_off_len, off_len;
-  char *off_enc = to_e_seed(seed_off, &off_len);
-  char *non_off_enc = to_e_seed(seed_no_off, &non_off_len);
+  uint8_t *off_enc = to_e_seed(alph, seed_off, &off_len);
+  uint8_t *non_off_enc = to_e_seed(alph, seed_no_off, &non_off_len);
 
   // An array of zeros is required for generating "difference columns":
   static int zero_vector[MAXALPH];
@@ -463,7 +466,7 @@ static void create_diff_cols (
  *
  * Print a representation of this seed_diffs object to the specified file.
  */
-extern void print_seed_diffs (
+void print_seed_diffs (
   SEED_DIFFS *diffs, ///< Seed diffs object
   FILE *out
 ) {
