@@ -542,13 +542,13 @@ void score_model_pssm(
         // Round the score(s) so sorting will be consistent across platforms.
         RND(score, RNDDIG, score);
         if (do_rc) RND(rc_score, RNDDIG, rc_score);
-        // Save position if best for sequence so far.
         if (do_rc && rc_score > score) {
           score = rc_score;
           strand = '-';
         } else {
           strand = '+';
         }
+        // Save position if best for sequence so far.
         if (score > best_score) {
           best_pos = pos;
           best_score = score;
@@ -843,6 +843,20 @@ void append_rc_to_multiseq (Multiseq *multiseq)
   multiseq->numofsequences *= 2;
 } // append_rc_to_multiseq
 
+void print_multiseq(
+  Multiseq *multiseq
+) {
+  Uint seqno;
+  for (seqno = 0; seqno < multiseq->numofsequences; seqno++) {
+    // Get the a pointer to the sequence and its length.
+    Uint seqstart = (seqno == 0) ? 0 : *(PEEKARRAY(&multiseq->markpos, seqno-1, Uint)) + 1;
+    Uint *markposptr = PEEKARRAY(&multiseq->markpos, seqno, Uint);
+    Uint seqend = (markposptr == NULL) ? multiseq->totallength : *markposptr;
+    Uint seqlen = seqend - seqstart;
+    fprintf(stderr, ">%d\n%*.*s\n", seqno, seqlen, seqlen,  multiseq->sequence + seqstart);
+  }
+} // print_multiseq
+
 //
 // Append the negative sequences to the end
 // of the positive sequences in a multisequence object.
