@@ -23,16 +23,18 @@ struct tree_t {
 			           genome is placed at the root of the tree.
 			        */
   int       num_children;       /* How many children does this leaf have? */
-  BOOLEAN_T has_length;         /* Does this node have a branch length? */
+  bool has_length;         /* Does this node have a branch length? */
   float     length;             /* Length of branch leading to this node. */
-  BOOLEAN_T has_descendants;    /* Has the number of descendents been 
+  bool has_descendants;    /* Has the number of descendents been 
 				                           calculated? */
   int       num_descendants;    /* Number of nodes below this in the tree. */
   struct tree_t* children[MAX_DEGREE];    /* Children of the current node. */
 };
 
 static void convert_to_uniform_star_tree_helper(TREE_T* the_tree, TREE_T* new_tree, double length);
-
+// for getting rid of gcc's annoying warn_unused_result
+static inline void ignore_value (int i) { (void) i; }
+static inline void ignore_ptr (void* p) { (void) p; } 
 /*************************************************************************
  * Allocate memory for a new tree.
  *************************************************************************/
@@ -42,9 +44,9 @@ TREE_T* allocate_tree()
   a_tree->label[0] = '\0';
   a_tree->key[0] = '\0';
   a_tree->num_children = 0;
-  a_tree->has_length = FALSE;
+  a_tree->has_length = false;
   a_tree->length = 0.0;
-  a_tree->has_descendants = FALSE;
+  a_tree->has_descendants = false;
   a_tree->num_descendants = 0;
   int i;
   for(i = 0; i < MAX_DEGREE; i++) {
@@ -69,15 +71,15 @@ static void check_null_tree
 /*************************************************************************
  * Is the given tree a leaf node?
  *************************************************************************/
-BOOLEAN_T is_leaf
+bool is_leaf
   (TREE_T * const a_tree)
 {
   check_null_tree(a_tree);
 
   if (a_tree->num_children == 0) {
-    return(TRUE);
+    return(true);
   }
-  return(FALSE);
+  return(false);
 }
 
 /*************************************************************************
@@ -94,7 +96,7 @@ char * get_label
 /*************************************************************************
  * Is there a label at the root of the given tree?
  *************************************************************************/
-BOOLEAN_T has_label
+bool has_label
   (TREE_T * const a_tree)
 {
   check_null_tree(a_tree);
@@ -136,9 +138,9 @@ void set_length
   }
 
   if (length > 0) {
-    a_tree->has_length = TRUE;
+    a_tree->has_length = true;
   } else {
-    a_tree->has_length = FALSE;
+    a_tree->has_length = false;
   }
   a_tree->length = length;
 }
@@ -158,7 +160,7 @@ int get_num_children
  * Compute and store the number of descendants of each node in a tree.
  **************************************************************************/
 static void compute_descendants
-  (TREE_T * a_tree, BOOLEAN_T force) 
+  (TREE_T * a_tree, bool force) 
 {
   int i_child;
   int num_descendants;
@@ -171,12 +173,12 @@ static void compute_descendants
   /* Base case: A leaf has only itself as a descendant. */
   if (is_leaf(a_tree)) {
     num_descendants = 1;
-    a_tree->has_descendants = FALSE;
+    a_tree->has_descendants = false;
   }
   else {
     /* Recursive case: Descendants of each child. */
     num_descendants = 0;
-    a_tree->has_descendants = TRUE;
+    a_tree->has_descendants = true;
     for (i_child = 0; i_child < a_tree->num_children; i_child++) {
       compute_descendants(get_nth_child(i_child, a_tree), force);
       num_descendants += get_num_descendants(get_nth_child(i_child, a_tree), force);
@@ -189,7 +191,7 @@ static void compute_descendants
  * Get the total number of leaf descendants under a given root node.
  *************************************************************************/
 int get_num_descendants
-  (TREE_T * const a_tree, BOOLEAN_T force)
+  (TREE_T * const a_tree, bool force)
 {
   check_null_tree(a_tree);
   compute_descendants(a_tree, force);
@@ -336,7 +338,7 @@ void add_child
 
   /* Update the number of descendants, if necessary. */
   if (a_tree->has_descendants) {
-    compute_descendants(a_child, FALSE);
+    compute_descendants(a_child, false);
     a_tree->num_descendants += a_child->num_descendants;
   }
   
@@ -369,8 +371,8 @@ TREE_T * get_nth_child
  *************************************************************************/
 void remove_nth_child
   (const VERBOSE_T verbosity,
-   const BOOLEAN free_child,		// free child node?
-   const BOOLEAN_T free_children, 	/* Boolean: Free children as well? */
+   const bool free_child,		// free child node?
+   const bool free_children, 	/* Boolean: Free children as well? */
    const int       n,
    TREE_T *        a_tree)
 {
@@ -415,12 +417,12 @@ static void process_length
   long int  ordzero;
   double    value;
   double    divisor;
-  BOOLEAN_T pointread; // Have we read a decimal point?
-  BOOLEAN_T minusread; // Have we read a minus sign?
+  bool pointread; // Have we read a decimal point?
+  bool minusread; // Have we read a minus sign?
 
   ordzero = '0';
-  pointread = FALSE;
-  minusread = FALSE;
+  pointread = false;
+  minusread = false;
   value = 0.0;
   divisor = 1.0;
   get_non_blank(tree_file, one_char);
@@ -429,9 +431,9 @@ static void process_length
   while (((unsigned long)digit <= 9) | (*one_char == '.') ||
          (*one_char == '-')) {
     if (*one_char == '.') {
-      pointread = TRUE;
+      pointread = true;
     } else if (*one_char == '-') {
-      minusread = TRUE;
+      minusread = true;
     } else {
       value = value * 10.0 + digit;
       if (pointread) {
@@ -457,7 +459,7 @@ static void read_node
  FILE *    tree_file,
  TREE_T ** a_tree)
 {
-  BOOLEAN_T last_child;
+  bool last_child;
   int i_child;
   int i_char;
 
@@ -468,7 +470,7 @@ static void read_node
   if (*one_char == '(') {
 
     // Read in the children, one by one.
-    last_child = FALSE;
+    last_child = false;
     i_child = 0;
     while (!last_child) {
       get_non_blank(tree_file, one_char);
@@ -481,7 +483,7 @@ static void read_node
 
       // Are we done with this set of children?
       if (*one_char == ')') {
-        last_child = TRUE;
+        last_child = true;
 
         // Allow labels on internal nodes.
         get_non_blank(tree_file, one_char);
@@ -506,7 +508,7 @@ static void read_node
 
 	  // Skip line breaks.
 	  if (is_eoln(tree_file)) {
-	    fscanf(tree_file, "%*[^\n]");
+	    ignore_value(fscanf(tree_file, "%*[^\n]"));
 	    getc(tree_file);
 	  }
 	  *one_char = getc(tree_file);
@@ -537,7 +539,7 @@ static void read_node
 
       // Skip line breaks.
       if (is_eoln(tree_file)) {
-        fscanf(tree_file, "%*[^\n]");
+        ignore_value(fscanf(tree_file, "%*[^\n]"));
         getc(tree_file);
       }
       *one_char = getc(tree_file);
@@ -552,7 +554,7 @@ static void read_node
   // Get the branch length.
   if (*one_char == ':') {
     process_length(one_char, tree_file, *a_tree);
-    (*a_tree)->has_length = TRUE;
+    (*a_tree)->has_length = true;
   }
 }
 
@@ -568,7 +570,7 @@ void read_tree
 
   get_non_blank(tree_file, &one_char);
   read_node(&one_char, tree_file, a_tree);
-  fscanf(tree_file, "%*[^\n]");
+  ignore_value(fscanf(tree_file, "%*[^\n]"));
   getc(tree_file);
 }
 
@@ -847,7 +849,7 @@ TREE_T* convert_to_uniform_star_tree(
 )
 {
   double d_tree = get_total_length(the_tree);
-  int N = get_num_descendants(the_tree, FALSE);
+  int N = get_num_descendants(the_tree, false);
   double d = d_tree/(N-1);		// new branch length
   // Find the node corresponding to the given label.
   TREE_T *new_tree = get_named_tree(the_tree, the_label);
@@ -857,7 +859,7 @@ TREE_T* convert_to_uniform_star_tree(
   // Convert the tree.
   convert_to_uniform_star_tree_helper(the_tree, new_tree, d);
   // Make sure the tree is kosher.
-  (void) get_num_descendants(the_tree, TRUE);
+  (void) get_num_descendants(the_tree, true);
 
   return(new_tree);
 } // convert_to_uniform_star_tree
@@ -885,7 +887,7 @@ static void convert_to_uniform_star_tree_helper(
       TREE_T* child = get_nth_child(i_child, the_tree);
       convert_to_uniform_star_tree_helper(child, new_tree, length);
     }
-    free_tree(FALSE, the_tree);		// free non-leaf
+    free_tree(false, the_tree);		// free non-leaf
   }
   return;
 } // convert_to_uniform_star_tree_helper
@@ -949,8 +951,8 @@ TREE_T* reroot_tree(
       }
       remove_nth_child(
         verbosity,
-        FALSE,				// Don't free child node
-        FALSE,				// Don't go recursive
+        false,				// Don't free child node
+        false,				// Don't go recursive
         i_child,
         the_tree
       );
@@ -967,7 +969,7 @@ TREE_T* reroot_tree(
         set_length(old_length, new_root);		// pass old length up to parent
       } else {
         // backed up to the old root
-        new_root->has_length = FALSE;		// new root has no length
+        new_root->has_length = false;		// new root has no length
       }
       return(new_root);
     }
@@ -997,7 +999,7 @@ static TREE_T* trim_tree_recursive
       if (verbosity > NORMAL_VERBOSE) {
 	fprintf(stderr, "Removing leaf (%s).\n", label);
       }
-      free_tree(FALSE, the_tree);
+      free_tree(false, the_tree);
       return(NULL);
     }
   } 
@@ -1053,7 +1055,7 @@ static TREE_T* trim_tree_recursive
       copy_tree(this_child, the_tree);
 
       // Free the child.
-      free_tree(TRUE, this_child);
+      free_tree(true, this_child);
 
       // Set the new branch length.
       set_length(new_length, the_tree);
@@ -1064,7 +1066,7 @@ static TREE_T* trim_tree_recursive
       if (verbosity >= HIGHER_VERBOSE) {
 	      fprintf(stderr, "No children.\n");
       }
-      free_tree(TRUE, the_tree);
+      free_tree(true, the_tree);
       return(NULL);
     }
   }
@@ -1076,7 +1078,7 @@ static TREE_T* trim_tree_recursive
  * Remove leaves from a given tree.
  *************************************************************************/
 TREE_T* trim_tree
-(BOOLEAN_T      allocate,
+(bool      allocate,
  TREE_T*        the_tree,
  STRING_LIST_T* the_leaves)
 {
@@ -1094,7 +1096,7 @@ TREE_T* trim_tree
   return_value = trim_tree_recursive(return_value, the_leaves);
 
   // Remove length from root (unless root is NULL).
-  if (return_value) return_value->has_length = FALSE;
+  if (return_value) return_value->has_length = false;
 
   return(return_value);
 }
@@ -1195,7 +1197,7 @@ RDB_MATRIX_T* get_leaf_distances(TREE_T*  const the_tree) {
  * Free dynamic memory used by a tree.
  *************************************************************************/
 void free_tree
-  (const BOOLEAN_T free_children, /* Free children as well? */
+  (const bool free_children, /* Free children as well? */
    TREE_T * a_tree)
 {
   int i_child;
@@ -1205,7 +1207,7 @@ void free_tree
   }
 
   /* Recursively free all the children in the tree. */
-  if (free_children == TRUE) {
+  if (free_children == true) {
     for (i_child = 0; i_child < a_tree->num_children; i_child++) {
       free_tree(free_children, a_tree->children[i_child]);
     }
@@ -1243,7 +1245,7 @@ int main
 
   // Define the usage message.
   char      usage[400] = "";
-  strcat(usage, "USAGE: tree [options] <tree file>\n");
+  strcat(usage, "Usage: tree [options] <tree file>\n");
   strcat(usage, "\n");
   strcat(usage, "   Options:\n");
   strcat(usage, "     -verbosity 1|2|3|4 (default = 2)\n");
@@ -1262,13 +1264,13 @@ int main
     }
 
     if (strcmp(option_name, "verbosity") == 0) {
-      verbosity = FALSE;
+      verbosity = false;
     }
   }
 
   // Read the single required argument.
   if (option_index + 1 != argc) {
-    fprintf(stderr, usage);
+    fprintf(stderr, "%s", usage);
     exit(1);
   }
   tree_filename = argv[option_index];
@@ -1283,7 +1285,7 @@ int main
   write_tree(the_tree, stdout);
 
   /* Tie up loose ends. */
-  free_tree(TRUE, the_tree);
+  free_tree(true, the_tree);
   return(0);
 }
 
