@@ -322,6 +322,12 @@ const char*  boolean_to_string(bool the_boolean);
 
 bool boolean_from_string(char* true_or_false);
 
+//
+// Convert null-terminated unsigned character string to character string.
+// Remember to free the returned char string.
+//
+char *uchar_to_char(unsigned char *ustr);
+
 /*************************************************************************
  * Writes a unicode codepoint in UTF-8 encoding to the start of the buffer
  * and return it, optionally record the length of the written code unit.
