@@ -1,4 +1,3 @@
-
 /******************************************************************************
  * Create a FASTA indicator handler.
  ******************************************************************************/
@@ -16,7 +15,6 @@ var FIH = function (container, indicator, popup, options, change_handler) {
 FIH.prototype = Object.create(FastaHandler.prototype);
 FIH.prototype.constructor = FIH;
 
-
 /******************************************************************************
  * Clears all the indicators.
  ******************************************************************************/
@@ -121,6 +119,148 @@ FIH.prototype.end = function () {
   this.change_handler();
 };
 
+/******************************************************************************
+ * Create a BED file upload indicator handler.
+ ******************************************************************************/
+var BEDIH = function (container, indicator, popup, options, change_handler) {
+  "use strict";
+  this.container = container;
+  this.indicator = indicator;
+  this.popup = popup;
+  this.change_handler = change_handler;
+  this.update_timer = null;
+  this.display_error = true;
+  this.display_warn = false;
+  BedHandler.call(this, options);
+};
+
+BEDIH.prototype = Object.create(BedHandler.prototype);
+BEDIH.prototype.constructor = BEDIH;
+
+/******************************************************************************
+ * Clears all the indicators.
+ ******************************************************************************/
+BEDIH.prototype.clear_indicators = function() {
+  "use strict";
+  if (this.update_timer) {
+    clearInterval(this.update_timer);
+    this.update_timer = null;
+  }
+  this.popup.innerHTML = "";
+  this.indicator.style.width = "0%";
+  substitute_classes(this.container, ["good", "error", "warning"], []);
+  this.display_type = 0;
+};
+
+BEDIH.prototype._update_display = function () {
+  "use strict";
+  var table, summary, i, j, msg;
+  var row, cell, i, ul, li;
+  
+  this.indicator.style.width = (this.fraction * 100) + "%";
+  if (!this.updated) return;
+
+  summary = this.summary();
+  if (summary.error) {
+    if (this.display_type != 3) {
+      substitute_classes(this.container, ["good", "warning"], ["error"]); 
+      this.display_type = 3;
+    }
+  } else if (summary.warning) {
+    if (this.display_type != 2) {
+      substitute_classes(this.container, ["good", "error"], ["warning"]);
+      this.display_type = 2;
+    }
+  } else {
+    if (this.display_type != 1) {
+      substitute_classes(this.container, ["warning", "error"], ["good"]);
+      this.display_type = 1;
+    }
+  }
+  table = document.createElement("table");
+  for (i = 0; i < summary.messages.length; i++) {
+    msg = summary.messages[i];
+    row = table.insertRow(table.rows.length);
+    row.className = (msg.is_error ? "error" : "warning");
+    cell = row.insertCell(row.cells.length);
+    cell.appendChild(document.createTextNode(msg.is_error ? "\u2718" : "\u26A0"));
+    cell = row.insertCell(row.cells.length);
+    cell.appendChild(document.createTextNode(msg.message));
+    if (typeof msg.reasons !== "undefined") {
+      ul = document.createElement("ul");
+      for (j = 0; j < msg.reasons.length; j++) {
+        li = document.createElement("li");
+        li.appendChild(document.createTextNode(msg.reasons[j]));
+        ul.appendChild(li);
+      }
+      cell.appendChild(ul);
+    }
+  }
+  
+  this.popup.innerHTML = "";
+  this.popup.appendChild(table);
+
+}
+
+BEDIH.prototype.configure = function (options) {
+  if (typeof options != "object" || options == null) options = {};
+  // configure maximum file size
+  if (typeof options.file_max == "number") {
+    this.max_file_size = options.file_max;
+  } else {
+    this.max_file_size = null; // 20 MB
+  }
+  // specify a maximum name length?
+  if (typeof options.max_name_len == "number" && options.max_name_len >= 1) {
+    this.max_name_length = options.max_name_len;
+  } else {
+    this.max_name_length = null;
+  }
+  // specify a maximum description length?
+  if (typeof options.max_desc_len == "number" && options.max_desc_len >= 1) {
+    this.max_desc_length = options.max_desc_len;
+  } else {
+    this.max_desc_length = null;
+  }
+  // specifiy the maximum number of loci
+  if (typeof options.max_loci == "number" && options.max_loci >= 1) {
+    this.max_loci = options.max_loci;
+  } else {
+    this.max_seq_count = null;
+  }
+}
+
+BEDIH.prototype.reset = function () {
+  "use strict";
+  BedHandler.prototype.reset.apply(this, arguments);
+  this.clear_indicators();
+};
+
+BEDIH.prototype.begin = function () {
+  "use strict";
+  var me;
+  BedHandler.prototype.begin.apply(this, arguments);
+  if (this.update_timer) clearInterval(this.update_timer);
+  me = this; // reference 'this' inside closure
+  this.update_timer = setInterval(function () { me._update_display();}, 100);
+  this.change_handler();
+};
+
+/******************************************************************************
+ *
+ ******************************************************************************/
+BEDIH.prototype.end = function () {
+  "use strict";
+  BedHandler.prototype.end.apply(this, arguments);
+  if (this.update_timer) {
+    clearInterval(this.update_timer);
+    this.update_timer = null;
+  }
+  this._update_display();
+  if (this.display_type == 1) this.clear_indicators();
+  this.change_handler();
+};
+
 /******************************************************************************
  * Construct the manager of the alphabet input.
  ******************************************************************************/
@@ -381,7 +521,7 @@ var SequenceInput = function(container, options) {
   this.text_dbh = new FIH(this.text_surround, this.text_indicator, this.text_popup, options, function() {
     me._fire_sequences_checked_event();
   });
-  // get the file related parts
+  // get the FASTA file related parts
   this.file_surround = this.container.querySelector("span.sequence_file");
   this.file_indicator = this.file_surround.querySelector("span.indicator");
   this.file_input = this.file_surround.querySelector("input");
@@ -389,6 +529,19 @@ var SequenceInput = function(container, options) {
   this.file_dbh = new FIH(this.file_surround, this.file_indicator, this.file_popup, options, function() {
     me._fire_sequences_checked_event();
   });
+  // get the BED file related parts
+  this.bed_file_surround = this.container.querySelector("span.bed_file");
+  if (this.bed_file_surround !== null) {
+    this.bed_file_indicator = this.bed_file_surround.querySelector("span.indicator");
+    this.bed_file_input = this.bed_file_surround.querySelector("input");
+    this.bed_file_popup = this.bed_file_surround.querySelector("div.popup");
+    this.bed_file_db_category = container.querySelector("select.bed_file_db.category");
+    this.bed_file_db_listing = container.querySelector("select.bed_file_db.listing");
+    this.bed_file_db_version = container.querySelector("select.bed_file_db.version");
+    this.bed_file_dbh = new BEDIH(this.bed_file_surround, this.bed_file_indicator, this.bed_file_popup, options, function() {
+      me._fire_sequences_checked_event();
+    });
+  }
   // get the database related parts
   this.db_listing = container.querySelector("select.sequence_db.listing");
   this.db_version = container.querySelector("select.sequence_db.version");
@@ -414,7 +567,7 @@ var SequenceInput = function(container, options) {
   }
 
   // create a list of submittable fields so we can disable the ones we are not using
-  this.submittables = [this.text_area, this.file_input];
+  this.submittables = [this.text_area, this.file_input, this.bed_file_input];
   if (this.db_listing != null) this.submittables.push(this.db_listing);
   if (this.db_version != null) this.submittables.push(this.db_version);
   if (this.db_priors != null) this.submittables.push(this.db_priors);
@@ -432,14 +585,16 @@ var SequenceInput = function(container, options) {
         new Blob([this.embed_data.value], {"type": "text\/plain"}));
   }
   // initialise
-  this._priors_filter_update();
+  this.priors_filter_update();
   this._source_update();
-  this._text_update();
-  this._file_update();
+  this.text_update();
+  this.file_update();
+  this.bed_file_update();
+  this._bed_file_db_category_update();
   // add listeners
   if (this.prior_filter != null) {
     this.prior_filter.addEventListener('click', function() {
-      me._priors_filter_update();
+      me.priors_filter_update();
       me._source_update();
     }, false);
   }
@@ -448,16 +603,39 @@ var SequenceInput = function(container, options) {
   }, false);
   // detect typing or pasting into motif text area
   this.text_area.addEventListener('input', function() {
-    me._text_update(); 
+    me.text_update(); 
   }, false);
   // detect file changes
   this.file_input.addEventListener('change', function() {
-    me._file_update();
+    me.file_update();
   }, false);
+  // detect BED file changes
+  if (typeof this.bed_file_input !== "undefined") {
+    this.bed_file_input.addEventListener('change', function() {
+      me.bed_file_update();
+    }, false);
+  }
+  // detect BED file db category changes
+  if (typeof this.bed_file_input !== "undefined") {
+    this.bed_file_db_category.addEventListener('change', function() {
+      me._bed_file_db_category_update();
+    }, false);
+  }
+  // detect BED db changes
+  if (this.bed_file_db_listing != null) {
+    this.bed_file_db_listing.addEventListener('change', function() {
+      me._load_bed_file_db_versions(me.bed_file_db_listing.value)
+    }, false);
+  }
+  if (this.bed_file_db_version != null) {
+    this.bed_file_db_version.addEventListener('change', function() {
+      me.bed_file_validate();
+    },false);
+  }
   // detect db changes
   if (this.db_listing != null) {
     this.db_listing.addEventListener('change', function() {
-      me._load_versions(+me.db_listing.value);
+      me._load_versions(me.db_listing.value);
     }, false);
   }
   if (this.db_version != null) {
@@ -535,6 +713,9 @@ SequenceInput.prototype.check = function(restrict_alphabets) {
     } else if (source == "file") {
       exists = this.file_input.value.length > 0;
       handler = this.file_dbh;
+    } else if (source == "bedfile") {
+      exists = this.bed_file_input.value.length > 0;
+      handler = this.bed_file_dbh;
     } else if (source == "embed") {
       exists = true;
       handler = this.embed_handler;
@@ -582,6 +763,10 @@ SequenceInput.prototype.changed = function() {
     if (this.text_area.value.length != 0) return true;
   } else if (source == "file") {
     if (this.file_input.value.length != 0) return true;
+  } else if (source == "bedfile") {
+    if (this.bed_file_input.value.length != 0) return true;
+    if (!this.bed_file_db_listing.options[this.bed_file_db_listing.selectedIndex].defaultSelected) return true;
+    if (!this.bed_file_db_version.options[this.bed_file_db_version.selectedIndex].defaultSelected) return true;
   } else if (source == "db") {
     if (!this.db_listing.options[this.db_listing.selectedIndex].defaultSelected) return true;
     if (!this.db_version.options[this.db_version.selectedIndex].defaultSelected) return true;
@@ -604,8 +789,12 @@ SequenceInput.prototype.reset = function() {
   this.file_dbh.reset();
   this.text_area.value = "";
   this.text_dbh.reset();
+  this.bed_file_input.value = "";
+  this.bed_file_dbh.reset();
   if (this.db_listing != null) this._clear_select(this.db_listing);
   if (this.db_version != null) this._clear_select(this.db_version);
+  if (this.bed_file_db_listing != null) this._clear_select(this.bed_file_db_listing);
+  if (this.bed_file_db_version != null) this._clear_select(this.bed_file_db_version);
   for (i = 0; i < this.source.options.length; i++) {
     opt = this.source.options[i];
     opt.selected = opt.defaultSelected;
@@ -625,6 +814,8 @@ SequenceInput.prototype.get_alphabets = function() {
     return this.text_dbh.guess_alphabets();
   } else if (source == "file") {
     return this.file_dbh.guess_alphabets();
+  } else if (source == "bedfile") {
+    return [AlphStd.DNA]; 
   } else if (source == "db") {
     return this.db_alphabets;
   } else if (source == "embed") {
@@ -688,17 +879,23 @@ SequenceInput.prototype._source_update = function() {
     id = parseInt(source, 10);
     source = "db";
   }
-  substitute_classes(this.container, ["noseq", "text", "file", "embed", "db"], [source]);
+  substitute_classes(this.container, ["noseq", "text", "file", "bedfile", "embed", "db"], [source]);
   for (i = 0; i < this.submittables.length; i++) {
-    this.submittables[i].disabled = true;
+    if(this.submittables[i]) {
+      this.submittables[i].disabled = true;
+    }
   }
   if (this.parser) this.parser.cancel();
   if (source == "text") {
     this.text_area.disabled = false;
-    this._text_validate();
+    this.text_validate();
   } else if (source == "file") {
     this.file_input.disabled = false;
-    this._file_validate();
+    this.file_validate();
+  } else if (source == "bedfile") {
+    this.bed_file_input.disabled = false;
+    this._bed_file_load_listings();
+    this.bed_file_validate();
   } else if (source == "db") {
     // the db hidden inputs will be enabled by the _load_listings command
     this._load_listings(id);
@@ -731,7 +928,7 @@ SequenceInput.prototype._fire_sequences_checked_event = function() {
 /******************************************************************************
  *
  ******************************************************************************/
-SequenceInput.prototype._text_validate = function() {
+SequenceInput.prototype.text_validate = function() {
   var file;
   if (this.parser) this.parser.cancel();
   if (this.text_area.value.length == 0) {
@@ -749,7 +946,7 @@ SequenceInput.prototype._text_validate = function() {
 /******************************************************************************
  *
  ******************************************************************************/
-SequenceInput.prototype._text_update = function() {
+SequenceInput.prototype.text_update = function() {
   "use strict";
   var nodes, me;
   nodes = document.createTextNode(this.text_area.value);
@@ -763,7 +960,7 @@ SequenceInput.prototype._text_update = function() {
     me = this;
     this.timer = window.setTimeout(function() {
       me.timer = null;
-      me._text_validate();
+      me.text_validate();
     }, 300);
   }
 };
@@ -771,7 +968,7 @@ SequenceInput.prototype._text_update = function() {
 /******************************************************************************
  *
  ******************************************************************************/
-SequenceInput.prototype._file_validate = function() {
+SequenceInput.prototype.file_validate = function() {
   var file;
   if (this.parser) this.parser.cancel();
   if (!(file = this.file_input.files[0])) {
@@ -788,13 +985,71 @@ SequenceInput.prototype._file_validate = function() {
 /******************************************************************************
  *
  ******************************************************************************/
-SequenceInput.prototype._file_update = function() {
+SequenceInput.prototype.bed_file_validate = function() {
+  var alph, file, sequence_id, sequences, sequence_index, version_opt;
+  if (this.parser) this.parser.cancel();
+  if (!(file = this.bed_file_input.files[0])) {
+    this.bed_file_dbh.clear_indicators();
+    substitute_classes(this.container, ["dna"], []);
+    return;
+  }
+  this.bed_file_dbh.configure(this.options);
+  version_opt = this.bed_file_db_version.options[this.bed_file_db_version.selectedIndex];
+  sequences = (version_opt != null ? JSON.parse(version_opt.getAttribute("data-sequences")) : {});
+  alph = this.alph_info.innerText;
+  if (sequences != null) {
+    sequence_id = sequences[alph].id;
+    sequence_index = load_sequence_index(sequence_id);
+    this.parser = new BedChecker(this.bed_file_dbh);
+    this.parser.process_file(file, sequence_index);
+  }
+};
+
+/******************************************************************************
+ *
+ ******************************************************************************/
+SequenceInput.prototype.file_update = function() {
   "use strict";
   if (this.source.value == "file") {
-    this._file_validate();
+    this.file_validate();
   }
 };
 
+/******************************************************************************
+ *
+ ******************************************************************************/
+SequenceInput.prototype.bed_file_update = function() {
+  "use strict";
+  if (this.source.value == "bedfile") {
+    this.bed_file_validate();
+  }
+};
+
+/******************************************************************************
+ *
+ ******************************************************************************/
+SequenceInput.prototype._bed_file_db_category_update = function() {
+  "use strict";
+  if (this.source.value == "bedfile") {
+    this._bed_file_load_listings();
+  }
+};
+
+/******************************************************************************
+ *
+ ******************************************************************************/
+SequenceInput.prototype._bed_file_db_listings_update = function() {
+  "use strict";
+  this._load_versions();
+};
+
+/******************************************************************************
+ *
+ ******************************************************************************/
+SequenceInput.prototype._bed_file_db_versions_update = function() {
+  "use strict";
+};
+
 /******************************************************************************
  *
  ******************************************************************************/
@@ -855,6 +1110,48 @@ SequenceInput.prototype._load_listings = function(category_id) {
   request.send();
 };
 
+/******************************************************************************
+ *
+ ******************************************************************************/
+SequenceInput.prototype._bed_file_load_listings = function() {
+  "use strict";
+  var request, url, i, id, me, bed_file_db_category, list, optgroup, selected_listing_id;
+  // so we can access in the inner function
+  me = this;
+  bed_file_db_category = this.bed_file_db_category.value;
+  if (/^\d+$/.test(bed_file_db_category)) {
+    id = parseInt(bed_file_db_category, 10);
+  }
+  list = this.bed_file_db_listing;
+  selected_listing_id = (list.value != null ? parseInt(list.value, 10) : null);
+  optgroup = list.getElementsByTagName("optgroup")[0];
+  this._clear_select(list);
+  // now send the request
+  url = "../db/sequences?category=" + id + "&alphabets=" + this._xml_alph + "&short=" + this._xml_short;
+  request = new XMLHttpRequest();
+  request.addEventListener("load", function(evt) {
+    var data, listing, i, opt;
+    data = JSON.parse(request.responseText);
+    // ensure the list is empty
+    me._clear_select(list);
+    // add the other options
+    for (i = 0; i < data.listings.length; i++) {
+      listing = data.listings[i];
+      opt = new Option(listing.name, listing.id);
+      optgroup.appendChild(opt);
+      // try to retain old selection
+      if (listing.id === selected_listing_id) opt.selected = true;
+    }
+    // re-enable the list
+    list.disabled = false;
+    if (list.value) {
+      me._load_bed_file_db_versions(parseInt(list.value, 10));
+    }
+  }, false);
+  request.open("GET", url, true);
+  request.send();
+};
+
 /******************************************************************************
  *
  ******************************************************************************/
@@ -908,6 +1205,52 @@ SequenceInput.prototype._load_versions = function(listing_id) {
   request.send();
 };
 
+/******************************************************************************
+ *
+ ******************************************************************************/
+SequenceInput.prototype._load_bed_file_db_versions = function(listing_id) {
+  "use strict";
+  var me, request, url, i, list, optgroup, selected_version_id;
+  // so we can access in the inner function
+  me = this;
+  list = this.bed_file_db_version;
+  selected_version_id = (list.value != null ? parseInt(list.value, 10) : null);
+  optgroup = list.getElementsByTagName("optgroup")[0];
+  this._clear_select(list);
+  // now send the request
+  url = "../db/sequences?listing=" + listing_id + "&alphabets=" + this._xml_alph + "&short=" + this._xml_short;
+  request = new XMLHttpRequest();
+  request.addEventListener("load", function(evt) {
+    var data, version, i, j, hasPriors, opt;
+    data = JSON.parse(request.responseText);
+    // ensure the list is empty
+    me._clear_select(list);
+    // add the other options
+    for (i = 0; i < data.versions.length; i++) {
+      version = data.versions[i];
+      hasPriors = false;
+      for (j = 0; j < version.alphabets.length; j++) {
+        if (version.sequences[version.alphabets[j]].priorCount > 0) {
+          hasPriors = true;
+          break;
+        }
+      }
+      opt = new Option(version.name, version.id);
+      opt.setAttribute("data-alphabets", JSON.stringify(version.alphabets));
+      opt.setAttribute("data-sequences", JSON.stringify(version.sequences));
+      opt.className = (hasPriors ? "" : "no_priors");
+      optgroup.appendChild(opt);
+      // try to retain old selection
+      if (version.id === selected_version_id) opt.selected = true;
+    }
+    // re-enable the list
+    list.disabled = false;
+    me._update_bed_file_db_alphabets();
+  }, false);
+  request.open("GET", url, true);
+  request.send();
+};
+
 /******************************************************************************
  *
  ******************************************************************************/
@@ -915,7 +1258,9 @@ SequenceInput.prototype._update_db_alphabets = function() {
   "use strict";
   var filter, opt, sequences;
   filter = this.prior_filter != null && this.prior_filter.checked;
-  opt = this.db_version.options[this.db_version.selectedIndex];
+  if (this.db_version) {
+    opt = this.db_version.options[this.db_version.selectedIndex];
+  }
   if (opt != null) {
     sequences = JSON.parse(opt.getAttribute("data-sequences"));
     this.db_alphabets = JSON.parse(opt.getAttribute("data-alphabets")).filter(
@@ -932,7 +1277,30 @@ SequenceInput.prototype._update_db_alphabets = function() {
 /******************************************************************************
  *
  ******************************************************************************/
-SequenceInput.prototype._priors_filter_update = function() {
+SequenceInput.prototype._update_bed_file_db_alphabets = function() {
+  "use strict";
+  var filter, opt, sequences;
+  filter = this.prior_filter != null && this.prior_filter.checked;
+  if (this.bed_file_db_version) {
+    opt = this.bed_file_db_version.options[this.bed_file_db_version.selectedIndex];
+  }
+  if (opt != null) {
+    sequences = JSON.parse(opt.getAttribute("data-sequences"));
+    this.bed_file_db_alphabets = JSON.parse(opt.getAttribute("data-alphabets")).filter(
+          function (value) { return !filter || sequences[value].priorCount > 0; }
+        ).map(
+          function (value) { return AlphStd[value]; }
+        );
+  } else {
+    this.bed_file_db_alphabets = [];
+  }
+  this._fire_sequences_checked_event();
+};
+
+/******************************************************************************
+ *
+ ******************************************************************************/
+SequenceInput.prototype.priors_filter_update = function() {
   "use strict";
   toggle_class(this.container, "filter_priors", this.prior_filter != null && this.prior_filter.checked);
 };
@@ -995,3 +1363,25 @@ SequenceInput.prototype._load_priors = function() {
     optgroup.appendChild(new Option("Awaiting motifs with matching alphabet...", ""));
   }
 }
+
+/*****************************************************************************
+ * 
+ ****************************************************************************/
+function load_sequence_index(sequence_id) {
+  "use strict";
+  var i;
+  var index = new Map();
+  // now send the request
+  var url = "../db/sequences?sequence_index=" + sequence_id;
+  var request = new XMLHttpRequest();
+  request.addEventListener("load", function(evt) {
+    var data;
+    data = JSON.parse(request.responseText);
+    for (i = 0; i < data.index_entries.length; i++) {
+      index.set(data.index_entries[i].chromosome, data.index_entries[i].length);
+    }
+  }, false);
+  request.open("GET", url, false);
+  request.send();
+  return index;
+}
