@@ -10,15 +10,23 @@ import java.util.Collection;
 /**
  * Calculate statistics on the loci
  */
-public final class LociStats implements LociInfo {
-  private boolean frozen;
-  private SampleStats stats;
+public class LociStats extends SequenceStats implements SequenceInfo, LociInfo {
 
   /**
    * Class constructor
    */
   public LociStats() {
-    this.stats = new SampleStats(true);
+    super();
+  }
+
+  /**
+   * Process the sequence and update the statistics
+   * @param sequence the complete sequence
+   * @see #addSeqPart(CharSequence)
+   * @see #endSeq()
+   */
+  public void addSeq(long length) {
+    stats.update(length);
   }
 
   /**
@@ -26,8 +34,7 @@ public final class LociStats implements LociInfo {
    * @return The total count of loci
    */
   public long getLociCount() {
-    //return this.stats.getCount();
-    return 0;
+    return stats.getCount();
   }
 
   @Override
@@ -35,6 +42,10 @@ public final class LociStats implements LociInfo {
     out.startObject();
     out.property("type", "loci");
     out.property("count", getLociCount());
+    out.property("min", getMinLength());
+    out.property("max", getMaxLength());
+    out.property("avg", getAverageLength());
+    out.property("total", getTotalLength());
     out.endObject();
   }
 }
