@@ -77,7 +77,7 @@ public final class SQL {
     "  avgLen REAL NOT NULL,\n" +
     "  stdDLen REAL NOT NULL,\n" +
     "  obsolete INTEGER DEFAULT 0,\n" +
-    "  UNIQUE (listingId, alphabet, edition)\n" +
+    "  UNIQUE (listingId, alphabet, edition, fileSeq)\n" +
     ")";
   /** Create a table to store information about a prior file */
   public static final String CREATE_TBL_PRIOR_FILE =
@@ -231,6 +231,15 @@ public final class SQL {
     "WHERE c.id = l.categoryId\n" +
     "GROUP BY c.id";
 
+  /** Select categories with indexed genomes*/
+  public static final String SELECT_INDEXED_GENOME_CATEGORIES =
+    "SELECT categoryId, c.name, COUNT(l.id), 0 AS has_priors\n" +
+    "FROM tblCategory AS c\n" +
+    "  INNER JOIN tblListing AS l\n" +
+    "  INNER JOIN tblSequenceFile AS s\n" +
+    "WHERE c.id = l.categoryId AND l.id = s.listingId AND s.fileSeqIndex IS NOT NULL\n" +
+    "GROUP BY c.id";
+
   /* the sequence databases sort the categories by name and filter by obsolete */
   public static final String SELECT_SEQUENCE_CATEGORIES =
       "SELECT c.id, c.name, COUNT(DISTINCT l.id), MAX(p.id) IS NOT NULL AS has_priors\n" +
@@ -303,7 +312,7 @@ public final class SQL {
     "SELECT id FROM tblSequenceFile WHERE fileSeq = ?";
 
   public static final String SELECT_SEQUENCE_FILES_OF_LISTING =
-    "SELECT s.id, s.alphabet, s.edition, s.version, s.description, s.fileSeq, s.fileBg,\n" +
+    "SELECT s.id, s.alphabet, s.edition, s.version, s.description, s.fileSeq, s.fileBg, s.fileSeqIndex,\n" +
     "    s.sequenceCount, s.totalLen, s.minLen, s.maxLen, s.avgLen, s.stdDLen, COUNT(p.id) AS priors_count\n" +
     "FROM tblSequenceFile AS s\n" +
     "  OUTER LEFT JOIN tblPriorFile AS p ON s.id = p.sequenceId\n" +
@@ -313,7 +322,7 @@ public final class SQL {
 
   /* used to return a sequence file result */
   public static final String SELECT_SEQUENCE_FILE_OF_LISTING =
-      "SELECT s.id, s.alphabet, s.edition, s.version, s.description, s.fileSeq, s.fileBg,\n" +
+      "SELECT s.id, s.alphabet, s.edition, s.version, s.description, s.fileSeq, s.fileBg, s.fileSeqIndex,\n" +
       "    s.sequenceCount, s.totalLen, s.minLen, s.maxLen, s.avgLen, s.stdDLen, COUNT(p.id) AS priors_count\n" +
       "FROM tblSequenceFile AS s\n" +
       "  OUTER LEFT JOIN tblPriorFile AS p ON s.id = p.sequenceId\n" +
@@ -337,10 +346,19 @@ public final class SQL {
     "WHERE c.name = ? AND l.name = ? AND s.alphabet & ? != 0 AND s.edition > ?\n" +
     "ORDER BY s.edition DESC LIMIT 1";
 
+  /** So we can check that the listed files actually exist */
+ public static final String SELECT_SEQUENCE_FILE_INDEX_BY_ID =
+     "SELECT fileSeqIndex FROM tblSequenceFile WHERE id = ?";
+
   /** So we can check that the listed files actually exist */
  public static final String SELECT_ALL_SEQUENCE_FILES =
      "SELECT id, alphabet, fileSeq, fileBg FROM tblSequenceFile";
 
+  /** So we can check that the listed files actually exist */
+ public static final String SELECT_SEQUENCE_FILES_BY_FILESEQ =
+     "SELECT id, alphabet, fileSeq, fileBg FROM tblSequenceFile WHERE fileSeq LIKE ?";
+
+
   /** So we can check that the listed files actually exist */
  public static final String SELECT_ALL_OBSOLETE_SEQUENCE_FILES =
      "SELECT fileSeq, fileBg FROM tblSequenceFile WHERE obsolete != 0";
@@ -353,10 +371,10 @@ public final class SQL {
     "SELECT sequenceId, filePrior, fileDist, biosample, assay, source, url, description\n" +
      "FROM tblPriorFile WHERE id = ?";
 
-  public static final String SELECT_ENSEMBL_DNA_FASTA_FILES =
+  public static final String SELECT_DNA_FASTA_FILES =
    "SELECT tblSequenceFile.id, tblSequenceFile.fileSeq, tblSequenceFile.fileSeqIndex\n" +
    "FROM tblCategory, tblListing, tblSequenceFile\n" +
-   "WHERE tblCategory.name LIKE 'Ensembl Genomes and Proteins' \n" +
+   "WHERE tblCategory.name LIKE '%Genomes%' \n" +
      "AND tblListing.CategoryId = tblCategory.id\n" +
      "AND tblSequenceFile.alphabet = 2\n" +
      "AND tblSequenceFile.listingId = tblListing.id";
