@@ -1,8 +1,6 @@
 package au.edu.uq.imb.memesuite.servlet;
 
-import au.edu.uq.imb.memesuite.data.Alph;
-import au.edu.uq.imb.memesuite.data.AlphabetDataSource;
-import au.edu.uq.imb.memesuite.data.SequenceDataSource;
+import au.edu.uq.imb.memesuite.data.*;
 import au.edu.uq.imb.memesuite.servlet.util.*;
 import au.edu.uq.imb.memesuite.template.HTMLSub;
 import au.edu.uq.imb.memesuite.template.HTMLTemplate;
@@ -99,6 +97,14 @@ public class Streme extends SubmitJob<Streme.Data> {
       if (order != null) addArgs(args, "-order", order);
       if (align != null) addArgs(args, "-align", align);
       if (negSeq != null) addArgs(args, "-n", negSeq.getName());
+      if (posSeq instanceof LociDataSource) {
+        addArgs(args, "-bedfile-pos");
+        addArgs(args, "-genome-pos", ((LociDataSource)posSeq).getGenomeFileName());
+      }
+      if (negSeq instanceof LociDataSource) {
+        addArgs(args, "-bedfile-neg");
+        addArgs(args, "-genome-neg", ((LociDataSource)negSeq).getGenomeFileName());
+      }
       addArgs(args, posSeq.getName());
       return args.toString();
     }
@@ -230,7 +236,7 @@ public class Streme extends SubmitJob<Streme.Data> {
       } else {
 	// p-value threshold
 	data.thresh = paramNumber(feedback, "<i>P</i>-value threshold", request,
-          "thresh", 0.0, 0.5, 0.05);
+          "pvt", 0.0, 0.5, 0.05);
       }
       if (paramBool(request, "order_enable")) {
         data.order = paramInteger(feedback, "Markov order", request, "order", 0, 4, 0);
