@@ -2,7 +2,10 @@ package au.edu.uq.imb.memesuite.servlet.util;
 
 import au.edu.uq.imb.memesuite.data.*;
 import au.edu.uq.imb.memesuite.db.*;
+import au.edu.uq.imb.memesuite.io.fasta.BedException;
 import au.edu.uq.imb.memesuite.io.fasta.FastaException;
+import au.edu.uq.imb.memesuite.io.fasta.FastaIndexException;
+import au.edu.uq.imb.memesuite.io.fasta.BedWriter;
 import au.edu.uq.imb.memesuite.io.fasta.FastaParser;
 import au.edu.uq.imb.memesuite.template.HTMLSub;
 import au.edu.uq.imb.memesuite.template.HTMLSubGenerator;
@@ -23,6 +26,7 @@ import java.util.logging.Logger;
 
 import static au.edu.uq.imb.memesuite.servlet.util.WebUtils.*;
 import static au.edu.uq.imb.memesuite.servlet.ConfigurationLoader.CACHE_KEY;
+import static au.edu.uq.imb.memesuite.servlet.ConfigurationLoader.CONFIG_KEY;
 import static au.edu.uq.imb.memesuite.servlet.ConfigurationLoader.SEQUENCE_DB_KEY;
 
 
@@ -38,6 +42,9 @@ public class ComponentSequences extends PageComponent {
   private String fieldName; // for feedback
   private HTMLTemplate title;
   private HTMLTemplate subtitle;
+  private boolean disableBed;
+  private boolean disableText;
+  private boolean disableFile;
   private boolean loadDBs;
   private boolean loadPriors;
   private boolean enableNoSeq;
@@ -65,6 +72,7 @@ public class ComponentSequences extends PageComponent {
     NOSEQ,
     TEXT,
     FILE,
+    BED,
     EMBED,
     DATABASE
   }
@@ -100,6 +108,37 @@ public class ComponentSequences extends PageComponent {
     }
   }
 
+  private static class IndexedGenomeCategoryTemplate extends HTMLSubGenerator<Category> {
+    private HTMLTemplate template;
+    private boolean selectFirst;
+    boolean selectable;
+    boolean identifiable;
+    boolean nameable;
+    boolean first;
+
+    public IndexedGenomeCategoryTemplate(DBList db, HTMLTemplate template, 
+      boolean selectFirst) throws SQLException {
+      super(db.getIndexedGenomeCategories());
+      this.template = template;
+      this.selectFirst = selectFirst;
+      selectable = template.containsSubtemplate("selected");
+      identifiable = template.containsSubtemplate("id");
+      nameable = template.containsSubtemplate("name");
+      first = true;
+    }
+
+    @Override
+    protected HTMLSub transform(Category item) {
+      HTMLSub category = template.toSub();
+      if (selectable && selectFirst && first) category.set("selected", "selected");
+      if (identifiable) category.set("id", item.getID());
+      if (nameable) category.set("name", item.getName());
+      category.set("no_priors", item.hasPriors() ? "" : "no_priors");
+      first = false;
+      return category;
+    }
+  }
+
   public ComponentSequences(ServletContext context, HTMLTemplate info) throws ServletException {
     this.context = context;
     HTMLTemplateCache cache = (HTMLTemplateCache)context.getAttribute(CACHE_KEY);
@@ -112,6 +151,9 @@ public class ComponentSequences extends PageComponent {
     type = getEnum(info, "alph_type", AlphType.class, AlphType.ANY_ALPHABET);
     shortOnly = info.containsSubtemplate("short_only");
     loadDBs = info.containsSubtemplate("enable_db");
+    disableText = info.containsSubtemplate("disable_text");
+    disableFile = info.containsSubtemplate("disable_file");
+    disableBed = info.containsSubtemplate("disable_bed");
     loadPriors = loadDBs && info.containsSubtemplate("enable_priors");
     enableNoSeq = info.containsSubtemplate("enable_noseq");
     defaultOption = getEnum(info, "default", DefaultOption.class, DefaultOption.FILE);
@@ -167,6 +209,9 @@ public class ComponentSequences extends PageComponent {
       case FILE:
         sequences.getSub("file_option").set("selected", "selected");
         break;
+      case BED:
+        sequences.getSub("bed_option").set("selected", "selected");
+        break;
       case EMBED:
         if (embedSeqs != null) sequences.getSub("embed_option").set("selected", "selected");
         break;
@@ -199,6 +244,24 @@ public class ComponentSequences extends PageComponent {
       sequences.empty("db_section");
     }
 
+    if (!disableBed && db != null) {
+      try {
+        HTMLTemplate genome_opt = tmplSequences.getSubtemplate("component").
+            getSubtemplate("bed_file_section").getSubtemplate("genome_options").getSubtemplate("genome_option");
+        sequences.getSub("bed_file_section").getSub("genome_options").set("genome_option",
+            new IndexedGenomeCategoryTemplate(db, genome_opt, selectFirstDB));
+        sequences.getSub("bed_file_section").set("prefix", prefix);
+        if (disableText) sequences.empty("text_option");
+        if (disableFile) sequences.empty("file_option");
+      } catch (SQLException e) {
+        logger.log(Level.SEVERE, "Error loading indexed genome categories", e);
+      }
+    }
+    else {
+      sequences.empty("bed_option");
+      sequences.empty("bed_file_section");
+    }
+
     StringWriter buf = new StringWriter();
     JsonWr jsonWr = new JsonWr(buf, 18);
     try {
@@ -265,9 +328,77 @@ public class ComponentSequences extends PageComponent {
     return ok;
   }
 
+  public LociDataSource getLoci(FileCoord.Name name,
+      HttpServletRequest request, FeedbackHandler feedback) throws ServletException, IOException {
+    Part part = request.getPart(prefix + "_bedfile");
+    if (part == null || part.getSize() == 0) {
+        feedback.whine("No " + fieldName + " provided.");
+        return null; // no loci submitted
+    }
+    name.setOriginalName(getPartFilename(part));
+    BufferedReader in = null;
+    FileWriter out = null;
+    File file = null;
+    BedWriter writer = null;
+    boolean success = false;
+
+    // Get genome file and genome index file for loci
+    long listingId = Long.parseLong(paramRequire(request, prefix + "_bed_file_db_listing"), 10);
+    long edition = Long.parseLong(paramRequire(request, prefix + "_bed_file_db_version"), 10);
+    SequenceDBList db = (SequenceDBList)context.getAttribute(SEQUENCE_DB_KEY);
+    if (db == null) {
+        throw new ServletException("Unable to access the sequence database.");
+    }
+    SequenceDB dbFile;
+    try {
+      Set<AlphStd> dbAlphs = type.getStandardAlphabets();
+      dbFile = db.getSequenceFile(listingId, edition, dbAlphs);
+    } catch (SQLException e) {
+      throw new ServletException(e);
+    }
+    MemeSuiteProperties msp = (MemeSuiteProperties)context.getAttribute(CONFIG_KEY);
+    String dbDir = msp.getDbDir().toString();
+    String indexPath = dbDir + "/fasta_databases/" + dbFile.getSeqIndexName();
+    
+    // Copy the BED from the Request into a file
+    // and check for validity
+    try {
+      in  = new BufferedReader(new InputStreamReader(part.getInputStream()));
+      file = File.createTempFile("uploaded_loci_", ".bed");
+      file.deleteOnExit();
+      out = new FileWriter(file);
+      writer = new BedWriter(out);
+      writer.checkAndCopyBedFile(feedback, indexPath, in);
+      try {in.close();} finally {in = null;}
+      try {out.close();} finally {out = null;}
+      success = true;
+    } 
+    catch (IOException e) {
+      feedback.whine("Error reading index file for sequence database. " + e.toString());
+    }
+    catch (BedException e) {
+      feedback.whine(e.toString());
+    }
+    catch (FastaIndexException e) {
+      feedback.whine(e.toString());
+    } finally {
+      closeQuietly(in);
+      closeQuietly(out);
+      if (file != null && !success) {
+        if (!file.delete()) file.deleteOnExit();
+      }
+    }
+    if (success) {
+      LociStats statistics = writer.getStatsRecorder();
+      return new LociDataSource(file, name, dbFile, statistics);
+    }
+    return null;
+  }
+
   /**
-   * Sequences come from 4 sources.
-   * They can be typed, uploaded, embedded or be a pre-existing database.
+   * Sequences come from 5 sources.
+   * They can be typed, uploaded, embedded or be a pre-existing database
+   * or a combination of an upload BED file and a pre-existing database.
    * Typed, embedded and uploaded sequences need to be preprocessed to calculate
    * statistics and to ensure they are valid.
    *
@@ -340,7 +471,11 @@ public class ComponentSequences extends PageComponent {
       }
       if (success) return new SequenceDataSource(file, name, statistics);
       return null;
-    } else { // database
+    } 
+    else if (source.equals("bedfile")) {
+      return  getLoci(name, request, feedback);
+    }
+    else { // database
       long listingId, edition;
       // databases only support standard alphabets so we have to convert any restriction alphabets into standard alphabets
       Set<AlphStd> dbAlphs;
@@ -355,13 +490,9 @@ public class ComponentSequences extends PageComponent {
       }
       try {
         listingId = Long.parseLong(paramRequire(request, prefix + "_db_listing"), 10);
-      } catch (NumberFormatException e) {
-        throw new ServletException("Expected the listing ID to be a number", e);
-      }
-      try {
         edition = Long.parseLong(paramRequire(request, prefix + "_db_version"), 10);
       } catch (NumberFormatException e) {
-        throw new ServletException("Expected the version ID to be a number", e);
+        throw new ServletException("Expected the listing ID to be a number", e);
       }
       // now get a sequence database that matches the listing, version and alphabet
       SequenceDBList db = (SequenceDBList)context.getAttribute(SEQUENCE_DB_KEY);
@@ -391,9 +522,11 @@ public class ComponentSequences extends PageComponent {
       return dbFile;
     }
   }
+
   /**
-   * Sequences come from 4 sources.
-   * They can be typed, uploaded, embedded or be a pre-existing database.
+   * Sequences come from 5 sources.
+   * They can be typed, uploaded, embedded or be a pre-existing database
+   * or a combination of an upload BED file and a pre-existing database.
    * Typed, embedded and uploaded sequences need to be preprocessed to calculate
    * statistics and to ensure they are valid.
    *
@@ -409,8 +542,9 @@ public class ComponentSequences extends PageComponent {
   }
 
   /**
-   * Sequences come from 4 sources.
-   * They can be typed, uploaded, embedded or be a pre-existing database.
+   * Sequences come from 5 sources.
+   * They can be typed, uploaded, embedded or be a pre-existing database
+   * or a combination of an upload BED file and a pre-existing database.
    * Typed, embedded and uploaded sequences need to be preprocessed to calculate
    * statistics and to ensure they are valid.
    *
@@ -428,8 +562,9 @@ public class ComponentSequences extends PageComponent {
   }
 
   /**
-   * Sequences come from 4 sources.
-   * They can be typed, uploaded, embedded or be a pre-existing database.
+   * Sequences come from 5 sources.
+   * They can be typed, uploaded, embedded or be a pre-existing database
+   * or a combination of an upload BED file and a pre-existing database.
    * Typed, embedded and uploaded sequences need to be preprocessed to calculate
    * statistics and to ensure they are valid.
    *
