@@ -1,20 +1,27 @@
 package au.edu.uq.imb.memesuite.updatedb;
 
+import au.edu.uq.imb.memesuite.data.AlphStd;
 import au.edu.uq.imb.memesuite.util.MultiSourceStatus;
 import au.edu.uq.imb.memesuite.util.Progress;
+import static au.edu.uq.imb.memesuite.db.SQL.*;
 import org.apache.commons.io.FileUtils;
 import org.apache.commons.net.ftp.FTP;
 import org.apache.commons.net.ftp.FTPClient;
 import org.apache.commons.net.ftp.FTPReply;
+import org.apache.commons.net.ftp.FTPFile;
 import org.sqlite.SQLiteDataSource;
 
 import java.io.*;
+import java.net.HttpURLConnection;
 import java.net.SocketTimeoutException;
+import java.net.URL;
+import java.net.URLConnection;
 import java.sql.Connection;
 import java.sql.PreparedStatement;
 import java.sql.ResultSet;
 import java.sql.SQLException;
 import java.util.LinkedList;
+import java.util.*;
 import java.util.List;
 import java.util.ListIterator;
 import java.util.Properties;
@@ -25,7 +32,12 @@ import java.util.logging.Logger;
 import java.util.regex.Matcher;
 import java.util.regex.Pattern;
 
-import static au.edu.uq.imb.memesuite.db.SQL.*;
+import org.json.simple.JSONObject;
+import org.json.simple.JSONArray;
+import org.json.simple.JSONValue;
+import org.json.simple.parser.ParseException;
+import org.json.simple.parser.JSONParser;
+
 
 /**
  * This is the base class of the sequence updaters. It implements shared functionality.
@@ -45,8 +57,17 @@ public abstract class SequenceUpdater implements Callable<Void> {
   private static final int REPLY_TIMEOUT = (int)TimeUnit.SECONDS.toMillis(30);
   private static final long KEEP_ALIVE_TIMEOUT = TimeUnit.MINUTES.toMillis(4);
 
+  protected static Pattern dnaFilePattern1 = Pattern.compile("^.*\\.dna\\.primary_assembly\\.fa\\.gz$");
+  protected static Pattern dnaFilePattern2 = Pattern.compile("^.*\\.dna\\.toplevel\\.fa\\.gz$");
+  protected static Pattern aaFilePattern = Pattern.compile("^.*\\.pep\\.all\\.fa\\.gz$");
+  protected static Pattern aiFilePattern = Pattern.compile("^.*\\.pep\\.abinitio\\.fa\\.gz$");
+  protected static Pattern releasePattern = Pattern.compile("/release-(\\d+)/");
+  protected static Pattern collectionPattern = Pattern.compile("^.*collection$");
+  protected static final int RETRY_COUNT = 3;
+  protected static final int ERROR_COUNT = 10;
 
-  public SequenceUpdater(String name, SQLiteDataSource dataSource,
+  public SequenceUpdater(String name, 
+      SQLiteDataSource dataSource,
       ReentrantReadWriteLock dbLock, File binDir, File dbDir,
       ExecutorService worker, MultiSourceStatus status) {
     if (binDir == null) throw new NullPointerException("binDir is null");
@@ -62,6 +83,166 @@ public abstract class SequenceUpdater implements Callable<Void> {
     progress = new Progress(status, name);
   }
 
+  // Read a JSON object from the Ensembl REST interface.
+  protected String ensemblRestRead(URL url) {
+    String output = "";
+    Reader reader = null;
+
+    try {
+      URLConnection connection = url.openConnection();
+      HttpURLConnection httpConnection = (HttpURLConnection)connection;
+
+      httpConnection.setRequestProperty("Content-Type", "application/json");
+
+      InputStream response = connection.getInputStream();
+      int responseCode = httpConnection.getResponseCode();
+
+      if (responseCode != 200) {
+        throw new RuntimeException("Response code was not 200. Detected response was " + responseCode);
+      }
+
+      // Read from the REST interface.
+      reader = new BufferedReader(new InputStreamReader(response, "UTF-8"));
+      StringBuilder builder = new StringBuilder();
+      char[] buffer = new char[8192];
+      int read;
+      while ((read = reader.read(buffer, 0, buffer.length)) > 0) {
+        builder.append(buffer, 0, read);
+      }
+      output = builder.toString();
+    } catch (IOException e) {
+     logger.log(Level.SEVERE, "URL exception.");
+    } finally {
+      if (reader != null) try {
+        reader.close();
+      } catch (IOException logOrIgnore) {
+        logOrIgnore.printStackTrace();
+      }
+    }
+
+    return output;
+  } // ensemblRestRead()
+
+protected List<EnsemblGenome> ensemblAvailableGenomes(
+  String genomeRelease, boolean abinitio, int retrieverId, String categoryName, 
+  String host, String rootDir, String restSubdir, Map<String, String> name2Subdir) {
+    progress.setTask("Querying available genomes using REST", 0, -1);
+    List<EnsemblGenome> genomes = new ArrayList<EnsemblGenome>();
+
+    // get a listing of species
+    try {
+      logger.log(Level.INFO, "Starting Update of " + categoryName);
+
+      // Create the REST query.
+      String server = "https://rest.ensembl.org";
+      String ext = "/info/genomes/division/" + restSubdir + "?";
+      URL url = new URL(server + ext);
+
+      // Query the REST interface.
+      String output = ensemblRestRead(url);
+
+      // Parse the JSON string.
+      JSONParser parser = new JSONParser();
+      try {
+        Object obj = parser.parse(output);
+        JSONArray array = (JSONArray) obj;
+        Iterator itr = array.iterator();
+        boolean first_genome = true;
+        while (itr.hasNext()) {
+          JSONObject jenome = (JSONObject) itr.next();
+          String displayName = jenome.get("display_name").toString();
+          logger.log(Level.INFO, "Display Name: " + displayName);
+          String scientificName = jenome.get("scientific_name").toString();
+          String ftpDirName = jenome.get("name").toString();
+          String path = rootDir;
+          // Add the collection name to the path.
+          String collection = name2Subdir.get(ftpDirName);
+          if (collection != null) { path += collection; }
+          path += ftpDirName;
+          String dnaPath = path + "/dna/";
+          String proteinPath = path + "/pep/";
+          logger.log(Level.FINE, "Adding Ensembl DNA and protein genomes for " + displayName);
+          if (! abinitio) {
+            genomes.add(new EnsemblGenome(genomeRelease, retrieverId, categoryName, displayName, ftpDirName, AlphStd.DNA, false, dnaPath, host, dnaFilePattern1, dnaFilePattern2));
+          }
+          genomes.add(new EnsemblGenome(genomeRelease, retrieverId, categoryName, displayName, ftpDirName, AlphStd.PROTEIN, abinitio, proteinPath, host, abinitio ? aiFilePattern : aaFilePattern));
+        }
+      } catch(ParseException pe) {
+        logger.log(Level.SEVERE, "position: " + pe.getPosition());
+      }
+    } catch (IOException e) {
+     logger.log(Level.SEVERE, "URL exception in ensemblAvailableGenomes.");
+    }
+    return genomes;
+  } // ensemblAvailableGenomes
+
+  //
+  // Make a Map of from the name of the species to its directory.
+  // The species directory MUST be either:
+  //   1) in the baseDir, or
+  //   2) in a subdirectory of it named "*_collection".
+  //
+  protected Map<String, String> makeMapName2Subdir(
+    FTPClient ftpClient, String rootDir
+  ) throws IOException {
+    Map<String, String> map = new HashMap<>();
+
+    FTPFile[] topFiles = ftpClient.listFiles(rootDir);
+    for (FTPFile aFile : topFiles) {
+      Matcher m = collectionPattern.matcher(aFile.getName());
+      if (m.find()) {
+        String collectionName = aFile.getName();
+        String subDir = rootDir + collectionName + "/";
+        FTPFile[] subFiles = ftpClient.listFiles(subDir);
+        for (FTPFile bFile : subFiles) {
+          // System.out.println(subDir + "\t" + bFile.getName());
+          map.put(bFile.getName(), collectionName + "/");
+        }
+      } else {
+        // System.out.println(rootDir + "\t" + aFile.getName());
+        map.put(aFile.getName(), "");
+      }
+    }
+    return map;
+  } // makeMapName2Subdir
+
+  protected boolean determineFtpSource(String host, FTPClient ftp, EnsemblGenome genome, int retryCount) throws IOException, InterruptedException {
+    for (int attempt = 1; attempt <= retryCount; attempt++) {
+      try {
+        // this could happen if we're retrying
+        if (!ftp.isConnected()) {
+          // try to recreate the connection
+          if (!loginFtp(ftp, host)) throw new IOException("Unable to log in to " + host);
+        }
+        return determineFtpSource(ftp, genome);
+      } catch (IOException e) {
+        logger.log(Level.WARNING, "Failed to determine file for " + genome + " " + attempt + " of " + retryCount, e);
+        if (attempt == retryCount) throw e;
+        if (ftp.isConnected()) {
+          try {
+            ftp.disconnect();
+          } catch (IOException e2) { /* ignore */ }
+        }
+        Thread.sleep(TimeUnit.SECONDS.toMillis(10));
+      }
+    }
+    return false;
+  }
+
+  protected boolean determineFtpSource(FTPClient ftp, EnsemblGenome genome) throws IOException {
+    progress.setTask("Determining full URL for " + genome, 0, -1);
+    PatternFileFilter filter = new PatternFileFilter(genome.getFilePattern());
+    logger.log(Level.INFO, "Looking for " + genome + " sequences at ftp://" + genome.getRemoteHost() + genome.getRemoteDir());
+    FTPFile sequenceFile = filter.best(ftp.listFiles(genome.getRemoteDir(), filter));
+    if (sequenceFile == null) {
+      logger.log(Level.WARNING, "Skipping ftp://" + genome.getRemoteHost() + " " + genome + " as no sequence found.");
+      return false;
+    }
+    logger.log(Level.INFO, "Using file " + sequenceFile.getName() + " for " + genome);
+    genome.setRemoteInfo(sequenceFile.getName(), sequenceFile.getSize());
+    return true;
+  } // determineFtpSource
+
   /**
    * Checks if a entry in the database exists for the combination of
    * category name, listing name, sequence alphabet and sequence edition. Can also
@@ -161,8 +342,7 @@ public abstract class SequenceUpdater implements Callable<Void> {
     return true;
   }
 
-  protected boolean downloadFtpSource(FTPClient ftp, FtpSource source, boolean supportResume, int retryCount)
-      throws IOException, ExecutionException, InterruptedException {
+  protected boolean downloadFtpSource(FTPClient ftp, FtpSource source, boolean supportResume, int retryCount) throws IOException, ExecutionException, InterruptedException {
     for (int attempt = 1; attempt <= retryCount; attempt++) {
       try {
         // this could happen if we're retrying
@@ -185,9 +365,7 @@ public abstract class SequenceUpdater implements Callable<Void> {
     return false;
   }
 
-  protected boolean downloadFtpSource(FTPClient ftp, FtpSource source,
-      boolean supportResume)
-      throws IOException, ExecutionException, InterruptedException {
+  protected boolean downloadFtpSource(FTPClient ftp, FtpSource source, boolean supportResume) throws IOException, ExecutionException, InterruptedException {
     checkWorkerTasks();
     progress.setTask("Downloading " + source, 0, source.getRemoteSize());
     // create the downloads directory if it does not exist
@@ -294,6 +472,78 @@ public abstract class SequenceUpdater implements Callable<Void> {
     return true;
   }
 
+  protected boolean downloadHttpSource(URL url, FtpSource source) throws IOException, ExecutionException, InterruptedException {
+    progress.setTask("Downloading " + source, 0, source.getRemoteSize());
+    // create the downloads directory if it does not exist
+    File downloadDir = new File(dbDir, "downloads");
+    if (downloadDir.exists()) {
+      if (!downloadDir.isDirectory()) {
+        throw new IOException("Unable to create download directory \"" +
+            downloadDir + "\" as a file with that name already exists!");
+      }
+    } else if (!downloadDir.mkdirs()) {
+      throw new IOException("Unable to create download directory \"" +
+          downloadDir + "\" as the mkdirs command failed!");
+    }
+    File target = new File(downloadDir, source.getLocalName());
+    logger.log(Level.INFO, "Downloading " + source + " from " +
+        source.getRemoteUrl() + " to \"" + target + "\".");
+
+    InputStream in = null;
+    FileOutputStream out = null;
+    try {
+      // Copy the file to the downloads directory.
+      URLConnection connection = url.openConnection();
+      HttpURLConnection httpConnection = (HttpURLConnection)connection;
+      httpConnection.setRequestProperty("Content-Type", "application/octet-stream");
+      InputStream response = connection.getInputStream();
+      int responseCode = httpConnection.getResponseCode();
+      if (responseCode != 200) {
+	throw new RuntimeException("Response code was not 200. Detected response was " + responseCode);
+      }
+      //char[] buffer = new char[8192];
+      byte[] buffer = new byte[2048];
+      int read;
+      //in = new BufferedReader(new InputStreamReader(response, "UTF-8"));
+      //in = new BufferedReader(new InputStreamReader(response, "UTF-8"));
+      in = response;
+      out = new FileOutputStream(target, false);
+      while ((read = in.read(buffer, 0, buffer.length)) > 0) {
+	out.write(buffer, 0, read);
+	progress.addTaskPartDone(read);
+      }
+    } finally {
+      if (in != null) {
+        try {
+          in.close();
+        } catch (IOException e) {
+          logger.log(Level.WARNING, "Ignoring failure to close HTTP InputStream.", e);
+        }
+      }
+      if (out != null) {
+        try {
+          out.close();
+        } catch (IOException e) {
+          logger.log(Level.WARNING, "Ignoring failure to close downloaded file OutputStream.", e);
+        }
+      }
+    }
+    boolean transferOk = true; // assume true until told otherwise
+    // check for success, if we somehow kept the connection
+    if (!transferOk) {
+      logger.log(Level.WARNING, "Transfer command failed (" + source + ").");
+      return false;
+    }
+    // check the file size
+    if (target.length() != source.getRemoteSize()) {
+      logger.log(Level.WARNING, "Transfered file size does not match remote (" + source + ").");
+      return false;
+    }
+    logger.log(Level.INFO, "Download of " + source + " complete.");
+    source.setSourceFile(target);
+    return true;
+  }
+
   public synchronized void checkWorkerTasks() throws ExecutionException, InterruptedException {
     if (Thread.currentThread().isInterrupted()) throw new InterruptedException();
     Future task;
@@ -339,13 +589,11 @@ public abstract class SequenceUpdater implements Callable<Void> {
   protected static Properties loadConf(Class<?> owner, File dbDir, String name) {
     Properties conf = new Properties();
     File confFile = new File(dbDir, "conf" + File.separator + name);
-    if (!confFile.exists()) {
-      try {
-        FileUtils.copyInputStreamToFile(owner.getResourceAsStream(name), confFile);
-      } catch (IOException e) {
-        logger.log(Level.SEVERE, "Failed to copy configuration file from defaults", e);
-        return conf;
-      }
+    try {
+      FileUtils.copyInputStreamToFile(owner.getResourceAsStream(name), confFile);
+    } catch (IOException e) {
+      logger.log(Level.SEVERE, "Failed to copy configuration file from defaults", e);
+      return conf;
     }
     try {
       conf.load(new FileInputStream(confFile));
@@ -385,7 +633,6 @@ public abstract class SequenceUpdater implements Callable<Void> {
     return defVal;
   }
 
-
   private class FtpCopier extends Thread {
     private InputStream in;
     private OutputStream out;
